﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Drawing;
using System.Drawing.Imaging;
using System.Runtime.InteropServices;
using System.Drawing.Drawing2D;
using System.IO;

namespace APO_BB_DK_v012013
{
    public static class ExtBitmap
    {
        private static Bitmap GetArgbCopy(Image sourceImage)
        {
            #region GetArgbCopy
            Bitmap bmpNew = new Bitmap(sourceImage.Width, sourceImage.Height, PixelFormat.Format32bppArgb);
            using (Graphics graphics = Graphics.FromImage(bmpNew))
            {
                graphics.DrawImage(sourceImage, new Rectangle(0, 0, bmpNew.Width, bmpNew.Height), new Rectangle(0, 0, bmpNew.Width, bmpNew.Height), GraphicsUnit.Pixel);
                graphics.Flush();
            }
            return bmpNew;
            #endregion
        }

        public static Bitmap CopyToSquareCanvas(this Bitmap sourceBitmap, int canvasWidthLenght)
        {
            #region  CopyToSquareCanvas
            float ratio = 1.0f;
            int maxSide = sourceBitmap.Width > sourceBitmap.Height ? sourceBitmap.Width : sourceBitmap.Height;
            ratio = (float)maxSide / (float)canvasWidthLenght;
            Bitmap bitmapResult = (sourceBitmap.Width > sourceBitmap.Height ? new Bitmap(canvasWidthLenght, (int)(sourceBitmap.Height / ratio)) : new Bitmap((int)(sourceBitmap.Width / ratio), canvasWidthLenght));
            using (Graphics graphicsResult = Graphics.FromImage(bitmapResult))
            {
                graphicsResult.CompositingQuality = CompositingQuality.HighQuality;
                graphicsResult.InterpolationMode = InterpolationMode.HighQualityBicubic;
                graphicsResult.PixelOffsetMode = PixelOffsetMode.HighQuality;
                graphicsResult.DrawImage(sourceBitmap, new Rectangle(0, 0, bitmapResult.Width, bitmapResult.Height), new Rectangle(0, 0, sourceBitmap.Width, sourceBitmap.Height), GraphicsUnit.Pixel);
                graphicsResult.Flush();
            }
            return bitmapResult;
            #endregion
        }

        public static Bitmap ArithmeticBlend(this Bitmap sourceBitmap, Bitmap blendBitmap, FunkcjeWieloarg.FunkcjeWieloargumentoweTyp calculationType)
        {
            #region ArithmeticBlend
            BitmapData sourceData = sourceBitmap.LockBits(new Rectangle(0, 0, sourceBitmap.Width, sourceBitmap.Height), ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
            byte[] pixelBuffer = new byte[sourceData.Stride * sourceData.Height];
            Marshal.Copy(sourceData.Scan0, pixelBuffer, 0, pixelBuffer.Length);
            sourceBitmap.UnlockBits(sourceData);
            BitmapData blendData = blendBitmap.LockBits(new Rectangle(0, 0, blendBitmap.Width, blendBitmap.Height), ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
            byte[] blendBuffer = new byte[blendData.Stride * blendData.Height];
            Marshal.Copy(blendData.Scan0, blendBuffer, 0, blendBuffer.Length);
            blendBitmap.UnlockBits(blendData);

            for (int k = 0; (k + 4 < pixelBuffer.Length) && (k + 4 < blendBuffer.Length); k += 4)
            {
                pixelBuffer[k] = FunkcjeWieloarg.Calculate(pixelBuffer[k], blendBuffer[k], calculationType);
                pixelBuffer[k + 1] = FunkcjeWieloarg.Calculate(pixelBuffer[k + 1], blendBuffer[k + 1], calculationType);
                pixelBuffer[k + 2] = FunkcjeWieloarg.Calculate(pixelBuffer[k + 2], blendBuffer[k + 2], calculationType);
            }

            Bitmap resultBitmap = new Bitmap(sourceBitmap.Width, sourceBitmap.Height);
            BitmapData resultData = resultBitmap.LockBits(new Rectangle(0, 0, resultBitmap.Width, resultBitmap.Height), ImageLockMode.WriteOnly, PixelFormat.Format32bppArgb);
            Marshal.Copy(pixelBuffer, 0, resultData.Scan0, pixelBuffer.Length);
            resultBitmap.UnlockBits(resultData);

            return resultBitmap;
            #endregion
        }

        public static Bitmap Bitonal(this Bitmap sourceBitmap, Color darkColor, Color lightColor, int threshold)
        {
            #region Bitonal - progowanie z kolorem
            BitmapData sourceData = sourceBitmap.LockBits(new Rectangle(0, 0, sourceBitmap.Width, sourceBitmap.Height), ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
            byte[] pixelBuffer = new byte[sourceData.Stride * sourceData.Height];
            Marshal.Copy(sourceData.Scan0, pixelBuffer, 0, pixelBuffer.Length);
            sourceBitmap.UnlockBits(sourceData);

            for (int k = 0; k + 4 < pixelBuffer.Length; k += 4)
            {
                if (pixelBuffer[k] + pixelBuffer[k + 1] + pixelBuffer[k + 2] <= threshold)
                {
                    pixelBuffer[k] = darkColor.B;
                    pixelBuffer[k + 1] = darkColor.G;
                    pixelBuffer[k + 2] = darkColor.R;
                }
                else
                {
                    pixelBuffer[k] = lightColor.B;
                    pixelBuffer[k + 1] = lightColor.G;
                    pixelBuffer[k + 2] = lightColor.R;
                }
            }
            Bitmap resultBitmap = new Bitmap(sourceBitmap.Width, sourceBitmap.Height);
            BitmapData resultData = resultBitmap.LockBits(new Rectangle(0, 0, resultBitmap.Width, resultBitmap.Height), ImageLockMode.WriteOnly, PixelFormat.Format32bppArgb);
            Marshal.Copy(pixelBuffer, 0, resultData.Scan0, pixelBuffer.Length);
            resultBitmap.UnlockBits(resultData);
            return resultBitmap;
            #endregion
        }

        public static Bitmap BitonalOdwrot(this Bitmap sourceBitmap, Color darkColor, Color lightColor, int threshold)
        {
            #region Bitonal - progowanie z kolorem odwrotne
            BitmapData sourceData = sourceBitmap.LockBits(new Rectangle(0, 0, sourceBitmap.Width, sourceBitmap.Height), ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
            byte[] pixelBuffer = new byte[sourceData.Stride * sourceData.Height];
            Marshal.Copy(sourceData.Scan0, pixelBuffer, 0, pixelBuffer.Length);
            sourceBitmap.UnlockBits(sourceData);

            for (int k = 0; k + 4 < pixelBuffer.Length; k += 4)
            {
                if (pixelBuffer[k] + pixelBuffer[k + 1] + pixelBuffer[k + 2] <= threshold)
                {
                    pixelBuffer[k] = lightColor.B;
                    pixelBuffer[k + 1] = lightColor.G;
                    pixelBuffer[k + 2] = lightColor.R;
                }
                else
                {
                    pixelBuffer[k] = darkColor.B;
                    pixelBuffer[k + 1] = darkColor.G;
                    pixelBuffer[k + 2] = darkColor.R;
                }
            }
            Bitmap resultBitmap = new Bitmap(sourceBitmap.Width, sourceBitmap.Height);
            BitmapData resultData = resultBitmap.LockBits(new Rectangle(0, 0, resultBitmap.Width, resultBitmap.Height), ImageLockMode.WriteOnly, PixelFormat.Format32bppArgb);
            Marshal.Copy(pixelBuffer, 0, resultData.Scan0, pixelBuffer.Length);
            resultBitmap.UnlockBits(resultData);
            return resultBitmap;
            #endregion
        }

        public static Bitmap CopyAsNegative(this Image sourceImage)
        {
            #region CopyAsNegative
            Bitmap bmpNew = GetArgbCopy(sourceImage);
            BitmapData bmpData = bmpNew.LockBits(new Rectangle(0, 0, sourceImage.Width, sourceImage.Height), ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
            IntPtr ptr = bmpData.Scan0;
            byte[] byteBuffer = new byte[bmpData.Stride * bmpNew.Height];
            Marshal.Copy(ptr, byteBuffer, 0, byteBuffer.Length);
            byte[] pixelBuffer = null;
            int pixel = 0;

            for (int k = 0; k < byteBuffer.Length; k += 4)
            {
                pixel = ~BitConverter.ToInt32(byteBuffer, k);
                pixelBuffer = BitConverter.GetBytes(pixel);
                byteBuffer[k] = pixelBuffer[0];
                byteBuffer[k + 1] = pixelBuffer[1];
                byteBuffer[k + 2] = pixelBuffer[2];
            }
            Marshal.Copy(byteBuffer, 0, ptr, byteBuffer.Length);
            bmpNew.UnlockBits(bmpData);
            bmpData = null;
            byteBuffer = null;

            return bmpNew;
            #endregion
        }

        public static Bitmap NegacjaRGB(this Bitmap sourceBitmap)
        {
            #region NegacjaRGB
            Bitmap bmpHSV = new Bitmap(sourceBitmap);
            BitmapData data = bmpHSV.LockBits(new System.Drawing.Rectangle(0, 0, bmpHSV.Width, bmpHSV.Height), ImageLockMode.ReadWrite, PixelFormat.Format32bppArgb);        
            int negR, negG, negB;

            for (int x = 0; x < sourceBitmap.Width; x++)
                for (int y = 0; y < sourceBitmap.Height; y++)
                {
                    Color col = sourceBitmap.GetPixel(x, y);
                    negR = 255 - col.R;
                    negG = 255 - col.G;
                    negB = 255 - col.B;
                    Color col1 = Color.FromArgb(col.A, negR, negG, negB);
                    sourceBitmap.SetPixel(x, y, col1);
                }
            bmpHSV.UnlockBits(data);
            return sourceBitmap;
            #endregion
        }

        public static Bitmap Model_blue(this Bitmap sourceBitmap)
        {
            #region NegacjaRGB
            Bitmap bmpHSV = new Bitmap(sourceBitmap);
            BitmapData data = bmpHSV.LockBits(new System.Drawing.Rectangle(0, 0, bmpHSV.Width, bmpHSV.Height), ImageLockMode.ReadWrite, PixelFormat.Format32bppArgb);
         //   int negR, negG, negB;

            for (int x = 0; x < sourceBitmap.Width; x++)
                for (int y = 0; y < sourceBitmap.Height; y++)
                {
                    Color col = sourceBitmap.GetPixel(x, y);
                    int k = (int)(0.299 * col.R + 0.587 * col.G + 0.114 * col.B);
           //         negR = col.B;
      
                    Color col1 = Color.FromArgb(col.A, k, k, k);
                    sourceBitmap.SetPixel(x, y, col1);
                }
            bmpHSV.UnlockBits(data);
            return sourceBitmap;
            #endregion
        }

        public static Bitmap ModelRGB_R(this Bitmap sourceBitmap)
        {
            #region NegacjaRGB
            Bitmap bmpHSV = new Bitmap(sourceBitmap);
            BitmapData data = bmpHSV.LockBits(new System.Drawing.Rectangle(0, 0, bmpHSV.Width, bmpHSV.Height), ImageLockMode.ReadWrite, PixelFormat.Format32bppArgb);
            //   int negR, negG, negB;

            for (int x = 0; x < sourceBitmap.Width; x++)
                for (int y = 0; y < sourceBitmap.Height; y++)
                {
                    Color col = sourceBitmap.GetPixel(x, y);
              //      int k = (int)(0.299 * col.R + 0.587 * col.G + 0.114 * col.B);
                    int  k = col.B;

                    Color col1 = Color.FromArgb(col.A, k, k, k);
                    sourceBitmap.SetPixel(x, y, col1);
                }
            bmpHSV.UnlockBits(data);
            return sourceBitmap;
            #endregion
        }

        public static Bitmap NegacjaHSV(this Bitmap sourceBitmap)
        {
            #region NegacjaHSV
            Bitmap bmpHSV = new Bitmap(sourceBitmap);
            BitmapData data = bmpHSV.LockBits(new System.Drawing.Rectangle(0, 0, bmpHSV.Width, bmpHSV.Height), ImageLockMode.ReadWrite, PixelFormat.Format32bppArgb);

            unsafe
            {
                byte* pBase = (byte*)data.Scan0.ToPointer();
                for (int x = 0; x < sourceBitmap.Width; x++)
                {
                    for (int y = 0; y < sourceBitmap.Height; y++)
                    {
                        double h, s, v;
                        hsv.ColorToHSV(sourceBitmap.GetPixel(x, y), out h, out s, out v);
                        v = 1.0f - v;
                        sourceBitmap.SetPixel(x, y, hsv.ColorFromHSV(h, s, v));
                    }
                }
                bmpHSV.UnlockBits(data);
                return sourceBitmap;
            }
            #endregion
        }
        
         public static Bitmap JasnoscHSV (this Bitmap sourceBitmap, float threshold)
        {
            #region JasnoscHSV
            Bitmap bmpHSV = new Bitmap(sourceBitmap);
             int new_height = sourceBitmap.Height;
             int new_width = sourceBitmap.Width;
             while (new_height > 300 || new_width > 300)
             {
                 new_height = (int)(new_height / 1.05f);
                 new_width = (int)(new_width / 1.05f);
             }
            BitmapData data = bmpHSV.LockBits(new System.Drawing.Rectangle(0, 0, bmpHSV.Width, bmpHSV.Height), ImageLockMode.ReadWrite, PixelFormat.Format32bppArgb);

            unsafe
            {
                byte* pBase = (byte*)data.Scan0.ToPointer();

                for (int x = 0; x < sourceBitmap.Width; x++)
                {
                    for (int y = 0; y < sourceBitmap.Height; y++)
                    {
                        double h, s, v;
                        hsv.ColorToHSV(sourceBitmap.GetPixel(x, y), out h, out s, out v);
                        v = v + ((double)threshold/255);
                        if (v < 0) v = 0.0f;
                        if (v > 1) v = 1.0f;

                        sourceBitmap.SetPixel(x, y, hsv.ColorFromHSV(h, s, v));
                    }
                }
                bmpHSV.UnlockBits(data);
                return sourceBitmap;
            }
            #endregion
        }

        public static Bitmap KontrastRGB(this Bitmap sourceBitmap, int threshold)
         {
             #region KontrastRGB
             Bitmap bmpHSV = new Bitmap(sourceBitmap);
            BitmapData data = bmpHSV.LockBits(new System.Drawing.Rectangle(0, 0, bmpHSV.Width, bmpHSV.Height), ImageLockMode.ReadWrite, PixelFormat.Format32bppArgb);
            double contrastLevel = (100 + threshold * 0.390625) / 100;
            double multi = (100 + threshold) / 100;
          //  unsafe
            {
            //    byte* pBase = (byte*)data.Scan0.ToPointer();
                //     BitmapProcessing.FastBitmap fobrazek_bitmapa = new BitmapProcessing.FastBitmap(sourceBitmap);
                //    fobrazek_bitmapa.LockImage();

                for (int x = 0; x < sourceBitmap.Width; x++)
                {
                    for (int y = 0; y < sourceBitmap.Height; y++)
                    {
                        double tmp = (sourceBitmap.GetPixel(x, y).R / 255) - 0.5;
                        tmp = tmp * multi + 0.5;
                    }
                }
                bmpHSV.UnlockBits(data);
                return sourceBitmap;
            }
             #endregion
         }

        public static Bitmap KontrastHSV(this Bitmap sourceBitmap, int threshold)
        {
            #region KontrastHSV
            Bitmap bmpHSV = new Bitmap(sourceBitmap);
            BitmapData data = bmpHSV.LockBits(new System.Drawing.Rectangle(0, 0, bmpHSV.Width, bmpHSV.Height), ImageLockMode.ReadWrite, PixelFormat.Format32bppArgb);
          //  double multi = (100 + threshold) / 100;
        //    double multi = (100 + threshold * 0.390625) / 100;
            //  double lmax = (double)(bmpHSV.Levels - 1);
            //--------------------------------------------------
       //     double multiplier = (100.0 + (double)value) / 100.0;
       //     double lmax = (double)(bmp.Levels - 1)                   
         //           double temp = ((double)bmp[x, y] / lmax) - 0.5;
          //          temp = temp * multiplier + 0.5;
          //          bmp[x, y] = (byte)Math.Max(0, Math.Min(bmp.Levels - 1, (int)(temp * lmax)));
              //-------------------------------- 


            unsafe
            {
                byte* pBase = (byte*)data.Scan0.ToPointer();

                for (int x = 0; x < sourceBitmap.Width; x++)
                {
                    for (int y = 0; y < sourceBitmap.Height; y++)
                    {
                        double h, s, v;
                        hsv.ColorToHSV(sourceBitmap.GetPixel(x, y), out h, out s, out v);
                        v = v + ((double)threshold / 255) - 0.5;
                   //     v = v * multi + 0.5;
                        //  v = (byte)Math.Max(0, Math.Min(255, (int)(v * 255.0)));
                        //   val = (int)(v*255.0);
                        if (v < 0) v = 0.0f;
                        if (v > 255) v = 1.0f;
                        sourceBitmap.SetPixel(x, y, hsv.ColorFromHSV(h, s, v));
                    }
                }
                bmpHSV.UnlockBits(data);
                return sourceBitmap;
            }
            #endregion
        }

        //------------
        public static Bitmap KontrastHSV_new(this Bitmap sourceBitmap, float threshold)
        {
            #region KontrastHSV new
            Bitmap bmpHSV = new Bitmap(sourceBitmap);
            int new_height = sourceBitmap.Height;
            int new_width = sourceBitmap.Width;
            while (new_height > 300 || new_width > 300)
            {
                new_height = (int)(new_height / 1.05f);
                new_width = (int)(new_width / 1.05f);
            }
            BitmapData data = bmpHSV.LockBits(new System.Drawing.Rectangle(0, 0, bmpHSV.Width, bmpHSV.Height), ImageLockMode.ReadWrite, PixelFormat.Format32bppArgb);

            unsafe
            {
                byte* pBase = (byte*)data.Scan0.ToPointer();

                for (int x = 0; x < sourceBitmap.Width; x++)
                {
                    for (int y = 0; y < sourceBitmap.Height; y++)
                    {
                        double h, s, v;
                        hsv.ColorToHSV(sourceBitmap.GetPixel(x, y), out h, out s, out v);
                        v = v + ((double)threshold / 255);
                        if (v < 0) v = 0.0f;
                        if (v > 1) v = 1.0f;

                        sourceBitmap.SetPixel(x, y, hsv.ColorFromHSV(h, s, v));
                    }
                }
                bmpHSV.UnlockBits(data);
                return sourceBitmap;
            }
            #endregion
        }

        public static Bitmap OdcienieRGB(this Bitmap sourceBitmap, float blueTint, float greenTint, float redTint)
        {
            #region OdcienieRGB
            BitmapData sourceData = sourceBitmap.LockBits(new Rectangle(0, 0, sourceBitmap.Width, sourceBitmap.Height), ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
            byte[] pixelBuffer = new byte[sourceData.Stride * sourceData.Height];
            Marshal.Copy(sourceData.Scan0, pixelBuffer, 0, pixelBuffer.Length);
            sourceBitmap.UnlockBits(sourceData);

            float blue = 0;
            float green = 0;
            float red = 0;

            for (int k = 0; k + 4 < pixelBuffer.Length; k += 4)
            {
                blue = pixelBuffer[k] + (255 - pixelBuffer[k]) * blueTint;
                green = pixelBuffer[k + 1] + (255 - pixelBuffer[k + 1]) * greenTint;
                red = pixelBuffer[k + 2] + (255 - pixelBuffer[k + 2]) * redTint;

                if (blue > 255)
                { blue = 255; }

                if (green > 255)
                { green = 255; }

                if (red > 255)
                { red = 255; }

                pixelBuffer[k] = (byte)blue;
                pixelBuffer[k + 1] = (byte)green;
                pixelBuffer[k + 2] = (byte)red;
            }
            Bitmap resultBitmap = new Bitmap(sourceBitmap.Width, sourceBitmap.Height);
            BitmapData resultData = resultBitmap.LockBits(new Rectangle(0, 0, resultBitmap.Width, resultBitmap.Height),ImageLockMode.WriteOnly, PixelFormat.Format32bppArgb);
            Marshal.Copy(pixelBuffer, 0, resultData.Scan0, pixelBuffer.Length);
            resultBitmap.UnlockBits(resultData);
            return resultBitmap;
            #endregion
        }
                   
        public static Bitmap Bineryzacja(this Bitmap sourceBitmap, int threshold)
        {
            #region Bineryzacja
            Bitmap bmpHSV = new Bitmap(sourceBitmap);
            BitmapData data = bmpHSV.LockBits(new System.Drawing.Rectangle(0, 0, bmpHSV.Width, bmpHSV.Height), ImageLockMode.ReadWrite, PixelFormat.Format32bppArgb);
            //     double multi = (100 + threshold) / 100;

            for (int x = 0; x < sourceBitmap.Width; x++)
            {
                for (int y = 0; y < sourceBitmap.Height; y++)
                {
                    Color px = sourceBitmap.GetPixel(x, y);
                    if ((int)((px.R * 0.3) + (px.G * 0.59) + (px.B * 0.11)) <= threshold)
                    {
                        Color ob = Color.FromArgb(px.A, 0, 0, 0);
                        bmpHSV.SetPixel(x, y, ob);
                    }
                    else
                    {
                        Color ob = Color.FromArgb(px.A, 255, 255, 255);
                        bmpHSV.SetPixel(x, y, ob);
                    }
                }
                bmpHSV.UnlockBits(data);    
            }           
            return sourceBitmap;
            #endregion
        }

        public static Bitmap CopyWithTransparency(this Image sourceImage, byte alphaComponent = 100)
        {
            #region CopyWithTransparency
            Bitmap bmpNew = GetArgbCopy(sourceImage);
            BitmapData bmpData = bmpNew.LockBits(new Rectangle(0, 0, sourceImage.Width, sourceImage.Height), ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
            IntPtr ptr = bmpData.Scan0;
            byte[] byteBuffer = new byte[bmpData.Stride * bmpNew.Height];
            Marshal.Copy(ptr, byteBuffer, 0, byteBuffer.Length);

            for (int k = 3; k < byteBuffer.Length; k += 4)
            {
                byteBuffer[k] = alphaComponent;
            }
            Marshal.Copy(byteBuffer, 0, ptr, byteBuffer.Length);
            bmpNew.UnlockBits(bmpData);
            bmpData = null;
            byteBuffer = null;

            return bmpNew;
            #endregion
        }

        public static Bitmap CopyAsGrayscale(this Image sourceImage)
        {
            #region CopyAsGrayscale
            Bitmap bmpNew = GetArgbCopy(sourceImage);
            BitmapData bmpData = bmpNew.LockBits(new Rectangle(0, 0, sourceImage.Width, sourceImage.Height), ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
            IntPtr ptr = bmpData.Scan0;
            byte[] byteBuffer = new byte[bmpData.Stride * bmpNew.Height];
            System.Runtime.InteropServices.Marshal.Copy(ptr, byteBuffer, 0, byteBuffer.Length);
            float rgb = 0;

            for (int k = 0; k < byteBuffer.Length; k += 4)
            {
                rgb = byteBuffer[k] * 0.11f;
                rgb += byteBuffer[k + 1] * 0.59f;
                rgb += byteBuffer[k + 2] * 0.3f;
                byteBuffer[k] = (byte)rgb;
                byteBuffer[k + 1] = byteBuffer[k];
                byteBuffer[k + 2] = byteBuffer[k];
                byteBuffer[k + 3] = 255;
            }
            Marshal.Copy(byteBuffer, 0, ptr, byteBuffer.Length);
            bmpNew.UnlockBits(bmpData);
            bmpData = null;
            byteBuffer = null;

            return bmpNew;
            #endregion
        }

        public static Bitmap DrawAsGrayscale(this Image sourceImage)
        {
            #region DrawAsGrayscale
            ColorMatrix colorMatrix = new ColorMatrix(new float[][] {    
                                                                    new float[] {.3f, .3f, .3f, 0, 0},
                                                                    new float[] {.59f, .59f, .59f, 0, 0},
                                                                    new float[] {.11f, .11f, .11f, 0, 0},
                                                                    new float[] {0, 0, 0, 1, 0},
                                                                    new float[] {0, 0, 0, 0, 1}
                                                                    });                                                        
            return ApplyColorMatrix(sourceImage, colorMatrix);
            #endregion
        }

        public static Bitmap DrawAsGrayscale_Red(this Image sourceImage)
        {
            #region DrawAsGrayscale_Red
            ColorMatrix colorMatrix = new ColorMatrix(new float[][] {
                                                                     new float[] {1, 1, 1, 0, 0}, //red
                                                                     new float[] {0, 0, 0, 0, 0},
                                                                     new float[] {0, 0, 0, 0, 0},
                                                                     new float[] {0, 0, 0, 1, 0},
                                                                     new float[] {0, 0, 0, 0, 1}
                                                                    });
             return ApplyColorMatrix(sourceImage, colorMatrix);
            #endregion
        }
       
        public static Bitmap DrawAsGrayscale_Green(this Image sourceImage)   
        {
            #region DrawAsGrayscale_Green
            ColorMatrix colorMatrix = new ColorMatrix(new float[][] {
                                                                     new float[] {0, 0, 0, 0, 0}, //red
                                                                     new float[] {1, 1, 1, 0, 0}, //green grey
                                                                     new float[] {0, 0, 0, 0, 0},
                                                                     new float[] {0, 0, 0, 1, 0},
                                                                     new float[] {0, 0, 0, 0, 1}
                                                                    });        // r, g, b
            return ApplyColorMatrix(sourceImage, colorMatrix);
            #endregion
        }

        public static Bitmap DrawAsGrayscale_Blue(this Image sourceImage)     
        {
            #region DrawAsGrayscale_Blue
            ColorMatrix colorMatrix = new ColorMatrix(new float[][] {
                                                                     new float[] {0, 0, 0, 0, 0}, //red
                                                                     new float[] {0, 0, 0, 0, 0},
                                                                     new float[] {1, 1, 1, 0, 0},
                                                                     new float[] {0, 0, 0, 1, 0},
                                                                     new float[] {0, 0, 0, 0, 1}
                                                                    });
            return ApplyColorMatrix(sourceImage, colorMatrix);
            #endregion
        }

        public static Bitmap DrawRGB(this Image sourceImage)
        {
            #region DrawRGB
            ColorMatrix colorMatrix = new ColorMatrix(new float[][] {
                                                                     new float[] {1, 0, 0, 0, 0}, //red
                                                                     new float[] {0, 1, 0, 0, 0},
                                                                     new float[] {0, 0, 1, 0, 0},
                                                                     new float[] {0, 0, 0, 1, 0},
                                                                     new float[] {0, 0, 0, 0, 1}
                                                                    });
            return ApplyColorMatrix(sourceImage, colorMatrix);
            #endregion
        }

        public static Bitmap DrawNegatywRGB(this Image sourceImage)
        {
            #region DrawNegatywRGB
            ColorMatrix colorMatrix = new ColorMatrix(new float[][] {
                                                                     new float[] {-1, 0, 0, 0, 0}, //red
                                                                     new float[] {0, -1, 0, 0, 0},
                                                                     new float[] {0, 0, -1, 0, 0},
                                                                     new float[] {0, 0, 0, 1, 0},
                                                                     new float[] {1, 1, 1, 1, 1}
                                                                    });
            return ApplyColorMatrix(sourceImage, colorMatrix);
            #endregion
        }

        private static Bitmap ApplyColorMatrix(Image sourceImage, ColorMatrix colorMatrix)
        {
            #region ApplyColorMatrix
            Bitmap bmp32BppSource = GetArgbCopy(sourceImage);
            Bitmap bmp32BppDest = new Bitmap(bmp32BppSource.Width, bmp32BppSource.Height, PixelFormat.Format32bppArgb);

            using (Graphics graphics = Graphics.FromImage(bmp32BppDest))
            {
                ImageAttributes bmpAttributes = new ImageAttributes();
                bmpAttributes.SetColorMatrix(colorMatrix);
                graphics.DrawImage(bmp32BppSource, new Rectangle(0, 0, bmp32BppSource.Width, bmp32BppSource.Height), 0, 0, bmp32BppSource.Width, bmp32BppSource.Height, GraphicsUnit.Pixel, bmpAttributes);
            }
            bmp32BppSource.Dispose();
            return bmp32BppDest;
            #endregion
        }

        public static Bitmap CopyAsSepiaTone(this Image sourceImage)
        {
            #region CopyAsSepiaTone
            Bitmap bmpNew = GetArgbCopy(sourceImage);
            BitmapData bmpData = bmpNew.LockBits(new Rectangle(0, 0, sourceImage.Width, sourceImage.Height), ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
            IntPtr ptr = bmpData.Scan0;
            byte[] byteBuffer = new byte[bmpData.Stride * bmpNew.Height];
            Marshal.Copy(ptr, byteBuffer, 0, byteBuffer.Length);
            byte maxValue = 255;
            float r = 0;
            float g = 0;
            float b = 0;

            for (int k = 0; k < byteBuffer.Length; k += 4)
            {
                r = byteBuffer[k] * 0.189f + byteBuffer[k + 1] * 0.769f + byteBuffer[k + 2] * 0.393f;
                g = byteBuffer[k] * 0.168f + byteBuffer[k + 1] * 0.686f + byteBuffer[k + 2] * 0.349f;
                b = byteBuffer[k] * 0.131f + byteBuffer[k + 1] * 0.534f + byteBuffer[k + 2] * 0.272f;

                byteBuffer[k + 2] = (r > maxValue ? maxValue : (byte)r);
                byteBuffer[k + 1] = (g > maxValue ? maxValue : (byte)g);
                byteBuffer[k] = (b > maxValue ? maxValue : (byte)b);
            }
            Marshal.Copy(byteBuffer, 0, ptr, byteBuffer.Length);
            bmpNew.UnlockBits(bmpData);
            bmpData = null;
            byteBuffer = null;

            return bmpNew;
            #endregion
        }
    
        public static Bitmap MedianFilter(this Bitmap sourceBitmap, int matrixSize, int bias = 0, bool grayscale = false)
        {
            #region MedianFilter
            BitmapData sourceData = sourceBitmap.LockBits(new Rectangle(0, 0, sourceBitmap.Width, sourceBitmap.Height), ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
            byte[] pixelBuffer = new byte[sourceData.Stride * sourceData.Height];
            byte[] resultBuffer = new byte[sourceData.Stride * sourceData.Height];
            Marshal.Copy(sourceData.Scan0, pixelBuffer, 0, pixelBuffer.Length);
            sourceBitmap.UnlockBits(sourceData);

            if (grayscale == true)
            {
                float rgb = 0;
                for (int k = 0; k < pixelBuffer.Length; k += 4)
                {
                    rgb = pixelBuffer[k] * 0.11f;
                    rgb += pixelBuffer[k + 1] * 0.59f;
                    rgb += pixelBuffer[k + 2] * 0.3f;

                    pixelBuffer[k] = (byte)rgb;
                    pixelBuffer[k + 1] = pixelBuffer[k];
                    pixelBuffer[k + 2] = pixelBuffer[k];
                    pixelBuffer[k + 3] = 255;
                }
            }
            int filterOffset = (matrixSize - 1) / 2;
            int calcOffset = 0;
            int byteOffset = 0;

            List<int> neighbourPixels = new List<int>();
            byte[] middlePixel;

            for (int offsetY = filterOffset; offsetY < sourceBitmap.Height - filterOffset; offsetY++)
            {
                for (int offsetX = filterOffset; offsetX < sourceBitmap.Width - filterOffset; offsetX++)
                {
                    byteOffset = offsetY * sourceData.Stride + offsetX * 4;
                    neighbourPixels.Clear();

                    for (int filterY = -filterOffset; filterY <= filterOffset; filterY++)
                    {
                        for (int filterX = -filterOffset; filterX <= filterOffset; filterX++)
                        {
                            calcOffset = byteOffset + (filterX * 4) + (filterY * sourceData.Stride);

                            neighbourPixels.Add(BitConverter.ToInt32(pixelBuffer, calcOffset));
                        }
                    }
                    neighbourPixels.Sort();
                    middlePixel = BitConverter.GetBytes(neighbourPixels[filterOffset]);
                    resultBuffer[byteOffset] = middlePixel[0];
                    resultBuffer[byteOffset + 1] = middlePixel[1];
                    resultBuffer[byteOffset + 2] = middlePixel[2];
                    resultBuffer[byteOffset + 3] = middlePixel[3];
                }
            }
            Bitmap resultBitmap = new Bitmap(sourceBitmap.Width, sourceBitmap.Height);
            BitmapData resultData = resultBitmap.LockBits(new Rectangle(0, 0, resultBitmap.Width, resultBitmap.Height), ImageLockMode.WriteOnly, PixelFormat.Format32bppArgb);
            Marshal.Copy(resultBuffer, 0, resultData.Scan0, resultBuffer.Length);
            resultBitmap.UnlockBits(resultData);
            return resultBitmap;
            #endregion
        }

        private static Bitmap ConvolutionFilter(Bitmap sourceBitmap, double[,] filterMatrix, double factor = 1, int bias = 0, bool grayscale = false)
        {
            #region ConvolutionFilter
            BitmapData sourceData = sourceBitmap.LockBits(new Rectangle(0, 0, sourceBitmap.Width, sourceBitmap.Height), ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
            byte[] pixelBuffer = new byte[sourceData.Stride * sourceData.Height];
            byte[] resultBuffer = new byte[sourceData.Stride * sourceData.Height];
            Marshal.Copy(sourceData.Scan0, pixelBuffer, 0, pixelBuffer.Length);
            sourceBitmap.UnlockBits(sourceData);

            if (grayscale == true)
            {
                float rgb = 0;
                for (int k = 0; k < pixelBuffer.Length; k += 4)
                {
                    rgb = pixelBuffer[k] * 0.11f;
                    rgb += pixelBuffer[k + 1] * 0.59f;
                    rgb += pixelBuffer[k + 2] * 0.3f;

                    pixelBuffer[k] = (byte)rgb;
                    pixelBuffer[k + 1] = pixelBuffer[k];
                    pixelBuffer[k + 2] = pixelBuffer[k];
                    pixelBuffer[k + 3] = 255;
                }
            }

            double blue = 0.0;
            double green = 0.0;
            double red = 0.0;

            int filterWidth = filterMatrix.GetLength(1);
            int filterHeight = filterMatrix.GetLength(0);
            int filterOffset = (filterWidth - 1) / 2;
            int calcOffset = 0;
            int byteOffset = 0;

            for (int offsetY = filterOffset; offsetY < sourceBitmap.Height - filterOffset; offsetY++)
            {
                for (int offsetX = filterOffset; offsetX < sourceBitmap.Width - filterOffset; offsetX++)
                {
                    blue = 0;
                    green = 0;
                    red = 0;

                    byteOffset = offsetY * sourceData.Stride + offsetX * 4;

                    for (int filterY = -filterOffset; filterY <= filterOffset; filterY++)
                    {
                        for (int filterX = -filterOffset; filterX <= filterOffset; filterX++)
                        {
                            calcOffset = byteOffset + (filterX * 4) + (filterY * sourceData.Stride);
                            blue += (double)(pixelBuffer[calcOffset]) * filterMatrix[filterY + filterOffset, filterX + filterOffset];
                            green += (double)(pixelBuffer[calcOffset + 1]) * filterMatrix[filterY + filterOffset, filterX + filterOffset];
                            red += (double)(pixelBuffer[calcOffset + 2]) * filterMatrix[filterY + filterOffset, filterX + filterOffset];
                        }
                    }
                    blue = factor * blue + bias;
                    green = factor * green + bias;
                    red = factor * red + bias;

                    if (blue > 255)
                    { blue = 255; }
                    else if (blue < 0)
                    { blue = 0; }

                    if (green > 255)
                    { green = 255; }
                    else if (green < 0)
                    { green = 0; }

                    if (red > 255)
                    { red = 255; }
                    else if (red < 0)
                    { red = 0; }

                    resultBuffer[byteOffset] = (byte)(blue);
                    resultBuffer[byteOffset + 1] = (byte)(green);
                    resultBuffer[byteOffset + 2] = (byte)(red);
                    resultBuffer[byteOffset + 3] = 255;
                }
            }
            Bitmap resultBitmap = new Bitmap(sourceBitmap.Width, sourceBitmap.Height);
            BitmapData resultData = resultBitmap.LockBits(new Rectangle(0, 0, resultBitmap.Width, resultBitmap.Height), ImageLockMode.WriteOnly, PixelFormat.Format32bppArgb);
            Marshal.Copy(resultBuffer, 0, resultData.Scan0, resultBuffer.Length);
            resultBitmap.UnlockBits(resultData);
            return resultBitmap;
            #endregion
        }

        public static Bitmap ConvolutionFilter(this Bitmap sourceBitmap, double[,] xFilterMatrix, double[,] yFilterMatrix, double factor = 1, int bias = 0, bool grayscale = false)
        {
            #region  ConvolutionFilter 2
            BitmapData sourceData = sourceBitmap.LockBits(new Rectangle(0, 0, sourceBitmap.Width, sourceBitmap.Height), ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
            byte[] pixelBuffer = new byte[sourceData.Stride * sourceData.Height];
            byte[] resultBuffer = new byte[sourceData.Stride * sourceData.Height];
            Marshal.Copy(sourceData.Scan0, pixelBuffer, 0, pixelBuffer.Length);
            sourceBitmap.UnlockBits(sourceData);

            if (grayscale == true)
            {
                float rgb = 0;
                for (int k = 0; k < pixelBuffer.Length; k += 4)
                {
                    rgb = pixelBuffer[k] * 0.11f;
                    rgb += pixelBuffer[k + 1] * 0.59f;
                    rgb += pixelBuffer[k + 2] * 0.3f;

                    pixelBuffer[k] = (byte)rgb;
                    pixelBuffer[k + 1] = pixelBuffer[k];
                    pixelBuffer[k + 2] = pixelBuffer[k];
                    pixelBuffer[k + 3] = 255;
                }
            }

            double blueX = 0.0;
            double greenX = 0.0;
            double redX = 0.0;

            double blueY = 0.0;
            double greenY = 0.0;
            double redY = 0.0;

            double blueTotal = 0.0;
            double greenTotal = 0.0;
            double redTotal = 0.0;

            int filterOffset = 1;
            int calcOffset = 0;

            int byteOffset = 0;

            for (int offsetY = filterOffset; offsetY < sourceBitmap.Height - filterOffset; offsetY++)
            {
                for (int offsetX = filterOffset; offsetX < sourceBitmap.Width - filterOffset; offsetX++)
                {
                    blueX = greenX = redX = 0;
                    blueY = greenY = redY = 0;
                    blueTotal = greenTotal = redTotal = 0.0;
                    byteOffset = offsetY * sourceData.Stride + offsetX * 4;

                    for (int filterY = -filterOffset; filterY <= filterOffset; filterY++)
                    {
                        for (int filterX = -filterOffset; filterX <= filterOffset; filterX++)
                        {
                            calcOffset = byteOffset + (filterX * 4) + (filterY * sourceData.Stride);
                            blueX += (double)(pixelBuffer[calcOffset]) * xFilterMatrix[filterY + filterOffset, filterX + filterOffset];
                            greenX += (double)(pixelBuffer[calcOffset + 1]) * xFilterMatrix[filterY + filterOffset, filterX + filterOffset];
                            redX += (double)(pixelBuffer[calcOffset + 2]) * xFilterMatrix[filterY + filterOffset, filterX + filterOffset];
                            blueY += (double)(pixelBuffer[calcOffset]) * yFilterMatrix[filterY + filterOffset, filterX + filterOffset];
                            greenY += (double)(pixelBuffer[calcOffset + 1]) * yFilterMatrix[filterY + filterOffset, filterX + filterOffset];
                            redY += (double)(pixelBuffer[calcOffset + 2]) * yFilterMatrix[filterY + filterOffset, filterX + filterOffset];
                        }
                    }

                    blueTotal = Math.Sqrt((blueX * blueX) + (blueY * blueY));
                    greenTotal = Math.Sqrt((greenX * greenX) + (greenY * greenY));
                    redTotal = Math.Sqrt((redX * redX) + (redY * redY));

                    if (blueTotal > 255)
                    { blueTotal = 255; }
                    else if (blueTotal < 0)
                    { blueTotal = 0; }

                    if (greenTotal > 255)
                    { greenTotal = 255; }
                    else if (greenTotal < 0)
                    { greenTotal = 0; }

                    if (redTotal > 255)
                    { redTotal = 255; }
                    else if (redTotal < 0)
                    { redTotal = 0; }

                    resultBuffer[byteOffset] = (byte)(blueTotal);
                    resultBuffer[byteOffset + 1] = (byte)(greenTotal);
                    resultBuffer[byteOffset + 2] = (byte)(redTotal);
                    resultBuffer[byteOffset + 3] = 255;
                }
            }

            Bitmap resultBitmap = new Bitmap(sourceBitmap.Width, sourceBitmap.Height);
            BitmapData resultData = resultBitmap.LockBits(new Rectangle(0, 0, resultBitmap.Width, resultBitmap.Height), ImageLockMode.WriteOnly, PixelFormat.Format32bppArgb);
            Marshal.Copy(resultBuffer, 0, resultData.Scan0, resultBuffer.Length);
            resultBitmap.UnlockBits(resultData);
            return resultBitmap;
            #endregion
        }

        public static Bitmap Laplacian3x3Filter(this Bitmap sourceBitmap, bool grayscale = true)
        {
            #region Laplacian3x3Filter
            Bitmap resultBitmap = ExtBitmap.ConvolutionFilter(sourceBitmap, Matrix.Laplacian3x3, 1.0, 0, grayscale);
            return resultBitmap;
            #endregion
        }

        public static Bitmap Laplacian5x5Filter(this Bitmap sourceBitmap, bool grayscale = true)
        {
            #region Laplacian5x5Filter
            Bitmap resultBitmap = ExtBitmap.ConvolutionFilter(sourceBitmap, Matrix.Laplacian5x5, 1.0, 0, grayscale);
            return resultBitmap;
            #endregion
        }

        public static Bitmap LaplacianOfGaussianFilter(this Bitmap sourceBitmap)
        {
            #region LaplacianOfGaussianFilter
            Bitmap resultBitmap = ExtBitmap.ConvolutionFilter(sourceBitmap, Matrix.LaplacianOfGaussian, 1.0, 0, true);
            return resultBitmap;
            #endregion
        }

        public static Bitmap Laplacian3x3OfGaussian3x3Filter(this Bitmap sourceBitmap)
        {
            #region Laplacian3x3OfGaussian3x3Filter
            Bitmap resultBitmap = ExtBitmap.ConvolutionFilter(sourceBitmap, Matrix.Gaussian3x3, 1.0 / 16.0, 0, true);
            resultBitmap = ExtBitmap.ConvolutionFilter(resultBitmap, Matrix.Laplacian3x3, 1.0, 0, false);
            return resultBitmap;
            #endregion
        }

        public static Bitmap Laplacian3x3OfGaussian5x5Filter1(this Bitmap sourceBitmap)
        {
            #region Laplacian3x3OfGaussian5x5Filter1
            Bitmap resultBitmap = ExtBitmap.ConvolutionFilter(sourceBitmap, Matrix.Gaussian5x5Type1, 1.0 / 159.0, 0, true);
            resultBitmap = ExtBitmap.ConvolutionFilter(resultBitmap, Matrix.Laplacian3x3, 1.0, 0, false);
            return resultBitmap;
            #endregion
        }

        public static Bitmap Laplacian3x3OfGaussian5x5Filter2(this Bitmap sourceBitmap)
        {
            #region Laplacian3x3OfGaussian5x5Filter2
            Bitmap resultBitmap = ExtBitmap.ConvolutionFilter(sourceBitmap, Matrix.Gaussian5x5Type2, 1.0 / 256.0, 0, true);
            resultBitmap = ExtBitmap.ConvolutionFilter(resultBitmap, Matrix.Laplacian3x3, 1.0, 0, false);
            return resultBitmap;
            #endregion
        }


        public static Bitmap Roberts(this Bitmap sourceBitmap)
        {
            #region Roberts
            Bitmap resultBitmap = ExtBitmap.ConvolutionFilter(sourceBitmap, Matrix.Roberts_1, 1.0 / 16.0, 0, true);
            resultBitmap = ExtBitmap.ConvolutionFilter(resultBitmap, Matrix.Roberts_2, 1.0, 0, false);
            return resultBitmap;
            #endregion
        }

        public static Bitmap Laplacian5x5OfGaussian3x3Filter(this Bitmap sourceBitmap)
        {
            #region Laplacian5x5OfGaussian3x3Filter
            Bitmap resultBitmap = ExtBitmap.ConvolutionFilter(sourceBitmap, Matrix.Gaussian3x3, 1.0 / 16.0, 0, true);
            resultBitmap = ExtBitmap.ConvolutionFilter(resultBitmap, Matrix.Laplacian5x5, 1.0, 0, false);
            return resultBitmap;
            #endregion
        }

        public static Bitmap Laplacian5x5OfGaussian5x5Filter1(this Bitmap sourceBitmap)
        {
            #region Laplacian5x5OfGaussian5x5Filter1
            Bitmap resultBitmap = ExtBitmap.ConvolutionFilter(sourceBitmap, Matrix.Gaussian5x5Type1, 1.0 / 159.0, 0, true);
            resultBitmap = ExtBitmap.ConvolutionFilter(resultBitmap, Matrix.Laplacian5x5, 1.0, 0, false);
            return resultBitmap;
            #endregion
        }

        public static Bitmap Laplacian5x5OfGaussian5x5Filter2(this Bitmap sourceBitmap)
        {
            #region Laplacian5x5OfGaussian5x5Filter2
            Bitmap resultBitmap = ExtBitmap.ConvolutionFilter(sourceBitmap, Matrix.Gaussian5x5Type2, 1.0 / 256.0, 0, true);
            resultBitmap = ExtBitmap.ConvolutionFilter(resultBitmap, Matrix.Laplacian5x5, 1.0, 0, false);
            return resultBitmap;
            #endregion
        }

        public static Bitmap Sobel3x3Filter(this Bitmap sourceBitmap, bool grayscale = true)
        {
            #region Sobel3x3Filter
            Bitmap resultBitmap = ExtBitmap.ConvolutionFilter(sourceBitmap, Matrix.Sobel3x3Horizontal, Matrix.Sobel3x3Vertical, 1.0, 0, grayscale);
            return resultBitmap;
            #endregion
        }

        public static Bitmap Roberts_Filter(this Bitmap sourceBitmap, bool grayscale = true)
        {
            #region Roberts_Filter
            Bitmap resultBitmap = ExtBitmap.ConvolutionFilter(sourceBitmap, Matrix.Roberts_1, Matrix.Roberts_2, 1.0, 0, grayscale);
            return resultBitmap;
            #endregion
        }

        public static Bitmap Sobel_Filter(this Bitmap sourceBitmap, bool grayscale = true)
        {
            #region Sobel_Filter
            Bitmap resultBitmap = ExtBitmap.ConvolutionFilter(sourceBitmap, Matrix.Sobela_1, Matrix.Sobela_2, 1.0, 0, grayscale);
            return resultBitmap;
            #endregion
        }

        public static Bitmap Prewitta_Filter(this Bitmap sourceBitmap, bool grayscale = true)
        {
            #region Prewitta_Filter
            Bitmap resultBitmap = ExtBitmap.ConvolutionFilter(sourceBitmap, Matrix.Prewitta_1, Matrix.Prewitta_2, 1.0, 0, grayscale);
            return resultBitmap;
            #endregion
        }

        public static Bitmap PrewittFilter(this Bitmap sourceBitmap, bool grayscale = true)
        {
            #region PrewittFilter
            Bitmap resultBitmap = ExtBitmap.ConvolutionFilter(sourceBitmap, Matrix.Prewitt3x3Horizontal, Matrix.Prewitt3x3Vertical, 1.0, 0, grayscale);
            return resultBitmap;
            #endregion
        }

        public static Bitmap KirschFilter(this Bitmap sourceBitmap, bool grayscale = true)
        {
            #region KirschFilter
            Bitmap resultBitmap = ExtBitmap.ConvolutionFilter(sourceBitmap, Matrix.Kirsch3x3Horizontal, Matrix.Kirsch3x3Vertical, 1.0, 0, grayscale);
            return resultBitmap;
            #endregion
        }

        public static Bitmap Prewitta_N(this Bitmap sourceBitmap, bool grayscale = true)
        {
            #region Prewitta_N
            Bitmap resultBitmap = ExtBitmap.ConvolutionFilter(sourceBitmap, Matrix.Prewitta_N, 1.0, 0, grayscale);
            return resultBitmap;
            #endregion
        }

        public static Bitmap Prewitta_NE(this Bitmap sourceBitmap, bool grayscale = true)
        {
            #region Prewitta_NE
            Bitmap resultBitmap = ExtBitmap.ConvolutionFilter(sourceBitmap, Matrix.Prewitta_NE, 1.0, 0, grayscale);
            return resultBitmap;
            #endregion
        }

        public static Bitmap Prewitta_E (this Bitmap sourceBitmap, bool grayscale = true)
        {
            #region Prewitta_E
            Bitmap resultBitmap = ExtBitmap.ConvolutionFilter(sourceBitmap, Matrix.Prewitta_E, 1.0, 0, grayscale);
            return resultBitmap;
            #endregion
        }

        public static Bitmap Prewitta_SE(this Bitmap sourceBitmap, bool grayscale = true)
        {
            #region Prewitta_SE
            Bitmap resultBitmap = ExtBitmap.ConvolutionFilter(sourceBitmap, Matrix.Prewitta_SE, 1.0, 0, grayscale);
            return resultBitmap;
            #endregion
        }

        public static Bitmap Prewitta_S(this Bitmap sourceBitmap, bool grayscale = true)
        {
            #region Prewitta_S
            Bitmap resultBitmap = ExtBitmap.ConvolutionFilter(sourceBitmap, Matrix.Prewitta_S, 1.0, 0, grayscale);
            return resultBitmap;
            #endregion
        }

        public static Bitmap Prewitta_SW(this Bitmap sourceBitmap, bool grayscale = true)
        {
            #region Prewitta_SW
            Bitmap resultBitmap = ExtBitmap.ConvolutionFilter(sourceBitmap, Matrix.Prewitta_SW, 1.0, 0, grayscale);
            return resultBitmap;
            #endregion
        }

        public static Bitmap Prewitta_W(this Bitmap sourceBitmap, bool grayscale = true)
        {
            #region Prewitta_W
            Bitmap resultBitmap = ExtBitmap.ConvolutionFilter(sourceBitmap, Matrix.Prewitta_W, 1.0, 0, grayscale);
            return resultBitmap;
            #endregion
        }

        public static Bitmap Prewitta_NW(this Bitmap sourceBitmap, bool grayscale = true)
        {
            #region Prewitta_NW
            Bitmap resultBitmap = ExtBitmap.ConvolutionFilter(sourceBitmap, Matrix.Prewitta_NW, 1.0, 0, grayscale);
            return resultBitmap;
            #endregion
        }

        public static Bitmap Kirscha_N(this Bitmap sourceBitmap, bool grayscale = true)
        {
            #region Kirscha_N
            Bitmap resultBitmap = ExtBitmap.ConvolutionFilter(sourceBitmap, Matrix.Kirscha_N, 1.0, 0, grayscale);
            return resultBitmap;
            #endregion
        }

        public static Bitmap Kirscha_NE(this Bitmap sourceBitmap, bool grayscale = true)
        {
            #region Kirscha_NE
            Bitmap resultBitmap = ExtBitmap.ConvolutionFilter(sourceBitmap, Matrix.Kirscha_NE, 1.0, 0, grayscale);
            return resultBitmap;
            #endregion
        }

        public static Bitmap Kirscha_E(this Bitmap sourceBitmap, bool grayscale = true)
        {
            #region Kirscha_E
            Bitmap resultBitmap = ExtBitmap.ConvolutionFilter(sourceBitmap, Matrix.Kirscha_E, 1.0, 0, grayscale);
            return resultBitmap;
            #endregion
        }

        public static Bitmap Kirscha_SE(this Bitmap sourceBitmap, bool grayscale = true)
        {
            #region Kirscha_SE
            Bitmap resultBitmap = ExtBitmap.ConvolutionFilter(sourceBitmap, Matrix.Kirscha_SE, 1.0, 0, grayscale);
            return resultBitmap;
            #endregion
        }

        public static Bitmap Kirscha_S(this Bitmap sourceBitmap, bool grayscale = true)
        {
            #region Kirscha_S
            Bitmap resultBitmap = ExtBitmap.ConvolutionFilter(sourceBitmap, Matrix.Kirscha_S, 1.0, 0, grayscale);
            return resultBitmap;
            #endregion
        }

        public static Bitmap Kirscha_SW(this Bitmap sourceBitmap, bool grayscale = true)
        {
            #region Kirscha_SW
            Bitmap resultBitmap = ExtBitmap.ConvolutionFilter(sourceBitmap, Matrix.Kirscha_SW, 1.0, 0, grayscale);
            return resultBitmap;
            #endregion
        }

        public static Bitmap Kirscha_W(this Bitmap sourceBitmap, bool grayscale = true)
        {
            #region Kirscha_W
            Bitmap resultBitmap = ExtBitmap.ConvolutionFilter(sourceBitmap, Matrix.Kirscha_W, 1.0, 0, grayscale);
            return resultBitmap;
            #endregion
        }

        public static Bitmap Kirscha_NW(this Bitmap sourceBitmap, bool grayscale = true)
        {
            #region Kirscha_NW
            Bitmap resultBitmap = ExtBitmap.ConvolutionFilter(sourceBitmap, Matrix.Kirscha_NW, 1.0, 0, grayscale);
            return resultBitmap;
            #endregion
        }

        public static Bitmap Sobel_H(this Bitmap sourceBitmap, bool grayscale = true)
        {
            #region Sobel_H
            Bitmap resultBitmap = ExtBitmap.ConvolutionFilter(sourceBitmap, Matrix.Sobel3x3Horizontal, 1.0, 0, grayscale);
            return resultBitmap;
            #endregion
        }

        public static Bitmap Sobel_V(this Bitmap sourceBitmap, bool grayscale = true)
        {
            #region Sobel_V
            Bitmap resultBitmap = ExtBitmap.ConvolutionFilter(sourceBitmap, Matrix.Sobel3x3Vertical, 1.0, 0, grayscale);
            return resultBitmap;
            #endregion
        }

        public static Bitmap Laplacian3x3_L1(this Bitmap sourceBitmap, bool grayscale = true)
        {
            #region Laplacian3x3_L1
            Bitmap resultBitmap = ExtBitmap.ConvolutionFilter(sourceBitmap, Matrix.Laplacian_L1, 1.0, 0, grayscale);
            return resultBitmap;
            #endregion
        }

        public static Bitmap Laplacian3x3_L2(this Bitmap sourceBitmap, bool grayscale = true)
        {
            #region Laplacian3x3_L2
            Bitmap resultBitmap = ExtBitmap.ConvolutionFilter(sourceBitmap, Matrix.Laplacian_L2, 1.0, 0, grayscale);
            return resultBitmap;
            #endregion
        }

        public static Bitmap Laplacian3x3_L3(this Bitmap sourceBitmap, bool grayscale = true)
        {
            #region Laplacian3x3_L3
            Bitmap resultBitmap = ExtBitmap.ConvolutionFilter(sourceBitmap, Matrix.Laplacian_L3, 1.0, 0, grayscale);
            return resultBitmap;
            #endregion
        }

        public static Bitmap Laplacian3x3_L4(this Bitmap sourceBitmap, bool grayscale = true)
        {
            #region Laplacian3x3_L4
            Bitmap resultBitmap = ExtBitmap.ConvolutionFilter(sourceBitmap, Matrix.Laplacian_L4, 1.0, 0, grayscale);
            return resultBitmap;
            #endregion
        }

        public static Bitmap Laplacian3x3_L5(this Bitmap sourceBitmap, bool grayscale = true)
        {
            #region Laplacian3x3_L5
            Bitmap resultBitmap = ExtBitmap.ConvolutionFilter(sourceBitmap, Matrix.Laplacian_L5, 1.0, 0, grayscale);
            return resultBitmap;
            #endregion
        }

        public static Bitmap Laplacian3x3_Horizontal(this Bitmap sourceBitmap, bool grayscale = true)
        {
            #region Laplacian3x3_Horizontal
            Bitmap resultBitmap = ExtBitmap.ConvolutionFilter(sourceBitmap, Matrix.Laplacian_Horizontal, 1.0, 0, grayscale);
            return resultBitmap;
            #endregion
        }

        public static Bitmap Laplacian3x3_Vertical(this Bitmap sourceBitmap, bool grayscale = true)
        {
            #region Laplacian3x3_Vertical
            Bitmap resultBitmap = ExtBitmap.ConvolutionFilter(sourceBitmap, Matrix.Laplacian_Vertical, 1.0, 0, grayscale);
            return resultBitmap;
            #endregion
        }

        public static Bitmap Laplacian3x3_Ukosny(this Bitmap sourceBitmap, bool grayscale = true)
        {
            #region Laplacian3x3_Ukosny
            Bitmap resultBitmap = ExtBitmap.ConvolutionFilter(sourceBitmap, Matrix.Laplacian_Ukosny, 1.0, 0, grayscale);
            return resultBitmap;
            #endregion
        }

        //http://dikamilo.net/entry/modele-barw-oraz-implementacja-w-c-sharp/
        public static Bitmap Contrast(this Bitmap sourceBitmap, int threshold)
        {
            #region Contrast
            BitmapData sourceData = sourceBitmap.LockBits(new Rectangle(0, 0, sourceBitmap.Width, sourceBitmap.Height), ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
            byte[] pixelBuffer = new byte[sourceData.Stride * sourceData.Height];
            Marshal.Copy(sourceData.Scan0, pixelBuffer, 0, pixelBuffer.Length);
            sourceBitmap.UnlockBits(sourceData);
            double contrastLevel = (100 + threshold*0.390625) / 100;
         //   double contrastLevel = Math.Pow((100.0 + threshold) / 100.0, 2);

            double blue = 0;
            double green = 0;
            double red = 0;

            for (int k = 0; k + 4 < pixelBuffer.Length; k += 4)
            {
                blue = ((((pixelBuffer[k] / 255.0) - 0.5) * contrastLevel) + 0.5) * 255.0;
                green = ((((pixelBuffer[k + 1] / 255.0) - 0.5) * contrastLevel) + 0.5) * 255.0;
                red = ((((pixelBuffer[k + 2] / 255.0) - 0.5) * contrastLevel) + 0.5) * 255.0;

                if (blue > 255)
                { blue = 255; }
                else if (blue < 0)
                { blue = 0; }

                if (green > 255)
                { green = 255; }
                else if (green < 0)
                { green = 0; }

                if (red > 255)
                { red = 255; }
                else if (red < 0)
                { red = 0; }

                pixelBuffer[k] = (byte)blue;
                pixelBuffer[k + 1] = (byte)green;
                pixelBuffer[k + 2] = (byte)red;
            }
            Bitmap resultBitmap = new Bitmap(sourceBitmap.Width, sourceBitmap.Height);
            BitmapData resultData = resultBitmap.LockBits(new Rectangle(0, 0, resultBitmap.Width, resultBitmap.Height), ImageLockMode.WriteOnly, PixelFormat.Format32bppArgb);
            Marshal.Copy(pixelBuffer, 0, resultData.Scan0, pixelBuffer.Length);
            resultBitmap.UnlockBits(resultData);
            return resultBitmap;
            #endregion
        }

        public static Bitmap Jasnosc(this Bitmap sourceBitmap, int threshold)
        {
            #region Jasnosc
            BitmapData sourceData = sourceBitmap.LockBits(new Rectangle(0, 0, sourceBitmap.Width, sourceBitmap.Height), ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
            byte[] pixelBuffer = new byte[sourceData.Stride * sourceData.Height];
            Marshal.Copy(sourceData.Scan0, pixelBuffer, 0, pixelBuffer.Length);
            sourceBitmap.UnlockBits(sourceData);
            //  double contrastLevel = Math.Pow((100.0 + threshold) / 100.0, 2);

            double blue = 0;
            double green = 0;
            double red = 0;

            for (int k = 0; k + 4 < pixelBuffer.Length; k += 4)
            {
                blue = pixelBuffer[k] + threshold * (pixelBuffer.Length / 256.0);
                green = pixelBuffer[k + 1] + threshold * (pixelBuffer.Length / 256.0);
                red = pixelBuffer[k + 2] + threshold * (pixelBuffer.Length / 256.0);

                if (blue < 0)
                { blue = 0; }
                else if (blue > pixelBuffer.Length - 1)
                {
                    blue = pixelBuffer.Length - 1;
                }

                if (green < 0)
                { green = 0; }
                else if (green > pixelBuffer.Length - 1)
                {
                    green = pixelBuffer.Length - 1;
                }

                if (red < 0)
                { red = 0; }
                else if (red > pixelBuffer.Length - 1)
                {
                    red = pixelBuffer.Length - 1;
                }

                pixelBuffer[k] = (byte)blue;
                pixelBuffer[k + 1] = (byte)green;
                pixelBuffer[k + 2] = (byte)red;
            }
            Bitmap resultBitmap = new Bitmap(sourceBitmap.Width, sourceBitmap.Height);
            BitmapData resultData = resultBitmap.LockBits(new Rectangle(0, 0, resultBitmap.Width, resultBitmap.Height), ImageLockMode.WriteOnly, PixelFormat.Format32bppArgb);
            Marshal.Copy(pixelBuffer, 0, resultData.Scan0, pixelBuffer.Length);
            resultBitmap.UnlockBits(resultData);
            return resultBitmap;
            #endregion
        }

        public static Bitmap Progowanie(this Bitmap sourceBitmap, int threshold)
        {
            #region Progowanie
            BitmapData sourceData = sourceBitmap.LockBits(new Rectangle(0, 0, sourceBitmap.Width, sourceBitmap.Height), ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
            byte[] pixelBuffer = new byte[sourceData.Stride * sourceData.Height];
            Marshal.Copy(sourceData.Scan0, pixelBuffer, 0, pixelBuffer.Length);
            sourceBitmap.UnlockBits(sourceData);

            float rgb = 0;

            for (int k = 0; k + 4 < pixelBuffer.Length; k += 4)
            {
                rgb = pixelBuffer[k] * 0.11f;
                rgb += pixelBuffer[k + 1] * 0.59f;
                rgb += pixelBuffer[k + 2] * 0.3f;                          
                
                if (pixelBuffer[k] <= threshold)
                {
                    rgb = pixelBuffer[k] = 0;
                }
                else rgb = pixelBuffer[k] = 255;
                
                 if (pixelBuffer[k+1] <= threshold)
                 {
                     rgb = pixelBuffer[k + 1] = 0;
                 }
                 else rgb = pixelBuffer[k + 1] = 255;

                if  (pixelBuffer[k+2] <= threshold)
                {
                    rgb = pixelBuffer[k + 2] = 0;
                }
                else rgb = pixelBuffer[k + 2] = 255;

                if (rgb > 255)
                { rgb = 255; }
                else if (rgb < 0)
                { rgb = 0; }

                pixelBuffer[k] = (byte)rgb;
                pixelBuffer[k + 1] = pixelBuffer[k];
                pixelBuffer[k + 2] = pixelBuffer[k];
                pixelBuffer[k + 3] = 255;
            }
            Bitmap resultBitmap = new Bitmap(sourceBitmap.Width, sourceBitmap.Height);
            BitmapData resultData = resultBitmap.LockBits(new Rectangle(0, 0, resultBitmap.Width, resultBitmap.Height), ImageLockMode.WriteOnly, PixelFormat.Format32bppArgb);
            Marshal.Copy(pixelBuffer, 0, resultData.Scan0, pixelBuffer.Length);
            resultBitmap.UnlockBits(resultData);
            return resultBitmap;
            #endregion
        }

        public static Bitmap ProgowanieOdwrotne(this Bitmap sourceBitmap, int threshold)
        {
            #region Progowanie odwrotne
            BitmapData sourceData = sourceBitmap.LockBits(new Rectangle(0, 0, sourceBitmap.Width, sourceBitmap.Height), ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
            byte[] pixelBuffer = new byte[sourceData.Stride * sourceData.Height];
            Marshal.Copy(sourceData.Scan0, pixelBuffer, 0, pixelBuffer.Length);
            sourceBitmap.UnlockBits(sourceData);

            float rgb = 0;

            for (int k = 0; k + 4 < pixelBuffer.Length; k += 4)
            {
                rgb = pixelBuffer[k] * 0.11f;
                rgb += pixelBuffer[k + 1] * 0.59f;
                rgb += pixelBuffer[k + 2] * 0.3f;

                if (pixelBuffer[k] <= threshold)
                {
                    rgb = pixelBuffer[k] = 255;
                }
                else rgb = pixelBuffer[k] = 0;

                if (pixelBuffer[k + 1] <= threshold)
                {
                    rgb = pixelBuffer[k + 1] = 255;
                }
                else rgb = pixelBuffer[k + 1] = 0;

                if (pixelBuffer[k + 2] <= threshold)
                {
                    rgb = pixelBuffer[k + 2] = 255;
                }
                else rgb = pixelBuffer[k + 2] = 0;

                if (rgb > 255)
                { rgb = 255; }
                else if (rgb < 0)
                { rgb = 0; }

                pixelBuffer[k] = (byte)rgb;
                pixelBuffer[k + 1] = pixelBuffer[k];
                pixelBuffer[k + 2] = pixelBuffer[k];
                pixelBuffer[k + 3] = 255;
            }
            Bitmap resultBitmap = new Bitmap(sourceBitmap.Width, sourceBitmap.Height);
            BitmapData resultData = resultBitmap.LockBits(new Rectangle(0, 0, resultBitmap.Width, resultBitmap.Height), ImageLockMode.WriteOnly, PixelFormat.Format32bppArgb);
            Marshal.Copy(pixelBuffer, 0, resultData.Scan0, pixelBuffer.Length);
            resultBitmap.UnlockBits(resultData);
            return resultBitmap;
            #endregion
        }


        public static Bitmap KorekcjaGammaRGB(this Bitmap sourceBitmap, float threshold)
        {
            #region Korekcja Gamma RGB
            BitmapData sourceData = sourceBitmap.LockBits(new Rectangle(0, 0, sourceBitmap.Width, sourceBitmap.Height), ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
            byte[] pixelBuffer = new byte[sourceData.Stride * sourceData.Height];
            Marshal.Copy(sourceData.Scan0, pixelBuffer, 0, pixelBuffer.Length);
            sourceBitmap.UnlockBits(sourceData);
            //double contrastLevel = Math.Pow((100.0 + threshold) / 100.0, 2);
        //    double war = threshold / 100;
                  //    if ((255 * Math.Pow(i / 255.0, 1 / gamma)) > 255)
        //    int pos = (int)((lmax * Math.Pow(i / lmax, 1.0 / value)) + 0.5);
          //  pos = Math.Min(Math.Max(pos, 0), bmp.Levels - 1);

            byte[] upo = new byte[pixelBuffer.Length];
            double lmax = (double)(pixelBuffer.Length - 1);

            double blue = 0;
            double green = 0;
            double red = 0;

            for (int i = 0; i < pixelBuffer.Length; i++)
            {
                int pos = (int)((lmax * Math.Pow(i / lmax, 1.0 / threshold)) + 0.5);
                pos = Math.Min(Math.Max(pos, 0), pixelBuffer.Length - 1);
                upo[i] = (byte)pos;
            }

             for (int k = 0; k + 4 < pixelBuffer.Length; k += 4)
            {
                blue = upo[pixelBuffer[k]];
                green = upo[pixelBuffer[k + 1]];
                red = upo[pixelBuffer[k + 2]];

                /*

                if ((255 * Math.Pow(k / 255.0, 1 / threshold)) > 255)
                {
                    blue = pixelBuffer[k] = 255;
                    green = pixelBuffer[k+1] = 255;
                    blue = pixelBuffer[k+2] = 255;                   
                }
                else
                {
                    blue = pixelBuffer[k] = (byte)(255 * Math.Pow(k / 255.0, 1 / threshold));
                    green = pixelBuffer[k + 1] = (byte)(255 * Math.Pow(k+1 / 255.0, 1 / threshold));
                    blue = pixelBuffer[k + 2] = (byte)(255 * Math.Pow(k +2/ 255.0, 1 / threshold)); 
                }
              //  wykres.Series[0].Points.Add(new DataPoint(i, LUT[i]));
            */

         //       int pos = (int)((lmax * Math.Pow(i / lmax, 1.0 / threshold)) + 0.5);
         //       pos = Math.Min(Math.Max(pos, 0),pixelBuffer.Length - 1);
         //       upo[i] = (byte)pos;
            
            /*
           
     
            */  /*  
                if (blue < 0)
                { blue = 0; }
                else if (blue > pixelBuffer.Length - 1)
                {
                    blue = pixelBuffer.Length - 1;
                }

                if (green < 0)
                { green = 0; }
                else if (green > pixelBuffer.Length - 1)
                {
                    green = pixelBuffer.Length - 1;
                }

                if (red < 0)
                { red = 0; }
                else if (red > pixelBuffer.Length - 1)
                {
                    red = pixelBuffer.Length - 1;
                }
             */

                pixelBuffer[k] = (byte)blue;
                pixelBuffer[k + 1] = (byte)green;
                pixelBuffer[k + 2] = (byte)red;
            }
            Bitmap resultBitmap = new Bitmap(sourceBitmap.Width, sourceBitmap.Height);
            BitmapData resultData = resultBitmap.LockBits(new Rectangle(0, 0, resultBitmap.Width, resultBitmap.Height), ImageLockMode.WriteOnly, PixelFormat.Format32bppArgb);
            Marshal.Copy(pixelBuffer, 0, resultData.Scan0, pixelBuffer.Length);
            resultBitmap.UnlockBits(resultData);
            return resultBitmap;
            #endregion
        }

        public static Bitmap Rozciagnie(this Bitmap sourceBitmap, int threshold, int threshold2)
        {
            #region Rozciaganie
            BitmapData sourceData = sourceBitmap.LockBits(new Rectangle(0, 0, sourceBitmap.Width, sourceBitmap.Height), ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
            byte[] pixelBuffer = new byte[sourceData.Stride * sourceData.Height];
            Marshal.Copy(sourceData.Scan0, pixelBuffer, 0, pixelBuffer.Length);
            sourceBitmap.UnlockBits(sourceData);

            float blue = 0;
            float green = 0;
            float red = 0;

            for (int k = 0; k + 4 < pixelBuffer.Length; k += 4)
            {
                if (threshold <= threshold2)
                {
                    if (pixelBuffer[k] < threshold)
                    {
                        blue = 0;
                    }
                    if (pixelBuffer[k + 1] < threshold)
                    {
                        green = 0;
                    }
                    if (pixelBuffer[k + 2] < threshold)
                    {
                        red = 0;
                    }
                    if (pixelBuffer[k] > threshold2)
                    {
                        blue = 255;
                    }
                    if (pixelBuffer[k + 1] > threshold2)
                    {
                        green = 255;
                    }
                    if (pixelBuffer[k + 2] > threshold2)
                    {
                        red = 255;
                    }
                    else if (threshold != threshold2)
                    {
                        blue = ((float)(255 * (pixelBuffer[k] - threshold))) / ((float)(threshold2 - threshold));
                        green = ((float)(255 * (pixelBuffer[k + 1] - threshold))) / ((float)(threshold2 - threshold));
                        red = ((float)(255 * (pixelBuffer[k + 2] - threshold))) / ((float)(threshold2 - threshold));
                    }
                }
                 
                if (threshold2 < threshold)
                {
                    if (pixelBuffer[k] < threshold2)
                    {
                        blue = 0;
                    }
                    if (pixelBuffer[k+1] < threshold2)
                    {
                        green = 0;
                    }
                    if (pixelBuffer[k+2] < threshold2)
                    {
                        red = 0;
                    }
                    if (pixelBuffer[k] > threshold)
                    {
                        blue = 255;
                    }
                    if (pixelBuffer[k + 1] > threshold)
                    {
                        green = 255;
                    }
                    if (pixelBuffer[k + 2] > threshold)
                    {
                        red = 255;
                    }
                    else
                    {
                      blue = ((float)(255 *(pixelBuffer[k] - threshold2))) / ((float)(threshold - threshold2));
                      green = ((float)(255 *(pixelBuffer[k+1] - threshold2))) / ((float)(threshold - threshold2));
                      red = ((float)(255 *(pixelBuffer[k+2] - threshold2))) / ((float)(threshold - threshold2));                 
                    }
                   
                }
             

                if (blue > 255)
                { blue = 255; }
                else if (blue < 0)
                { blue = 0; }

                if (green > 255)
                { green = 255; }
                else if (green < 0)
                { green = 0; }

                if (red > 255)
                { red = 255; }
                else if (red < 0)
                { red = 0; }

                pixelBuffer[k] = (byte)blue;
                pixelBuffer[k + 1] = (byte)green;
                pixelBuffer[k + 2] = (byte)red;

            }
            Bitmap resultBitmap = new Bitmap(sourceBitmap.Width, sourceBitmap.Height);
            BitmapData resultData = resultBitmap.LockBits(new Rectangle(0, 0, resultBitmap.Width, resultBitmap.Height), ImageLockMode.WriteOnly, PixelFormat.Format32bppArgb);
            Marshal.Copy(pixelBuffer, 0, resultData.Scan0, pixelBuffer.Length);
            resultBitmap.UnlockBits(resultData);
            return resultBitmap;
            #endregion
        }


        public static Bitmap RozciagnieHSV ( this Bitmap sourceBitmap, int threshold, int threshold2)
        {
         //   BitmapData sourceData = sourceBitmap.LockBits(new Rectangle(0, 0, sourceBitmap.Width, sourceBitmap.Height), ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
       //     byte[] pixelBuffer = new byte[sourceData.Stride * sourceData.Height];
      //      Marshal.Copy(sourceData.Scan0, pixelBuffer, 0, pixelBuffer.Length);
       //     sourceBitmap.UnlockBits(sourceData);


            Bitmap sourceBitmap1 = new Bitmap(sourceBitmap);
          BitmapProcessing.FastBitmap sourceBitmap2 = new BitmapProcessing.FastBitmap(sourceBitmap);
          sourceBitmap2.LockImage();



            for (int x = 0; x < sourceBitmap1.Width; x++)
            {
                for (int y = 0; y < sourceBitmap1.Height; y++)
                {
                    double h, s, v;
                    hsv.ColorToHSV(sourceBitmap2.GetPixel(x, y), out h, out s, out v);
                    double wartosc = v;
                    if (wartosc < ((double)threshold) / 255)
                    {
                        sourceBitmap2.SetPixel(x, y, hsv.ColorFromHSV(h, s, 0));
                    }
                    else if (wartosc > ((double)threshold2) / 255)
                    {
                        sourceBitmap2.SetPixel(x, y, hsv.ColorFromHSV(h, s, 1));
                    }
                    else
                    {
                        double tmp = wartosc;
                        if (threshold != threshold2)
                        {
                            tmp = (((wartosc - ((double)threshold/255.0f)))) / (((float)(threshold2 - threshold))/255.0f);
                        }

                        sourceBitmap2.SetPixel(x, y, hsv.ColorFromHSV(h, s, tmp));
                    }
                }
            }
        ////    Bitmap resultBitmap = new Bitmap(sourceBitmap.Width, sourceBitmap.Height);
        //    BitmapData resultData = resultBitmap.LockBits(new Rectangle(0, 0, resultBitmap.Width, resultBitmap.Height), ImageLockMode.WriteOnly, PixelFormat.Format32bppArgb);
       //     Marshal.Copy(pixelBuffer, 0, resultData.Scan0, pixelBuffer.Length);
       //     resultBitmap.UnlockBits(resultData);
       //     return resultBitmap;
            sourceBitmap2.UnlockImage();
            return sourceBitmap;        
        }


        public static Bitmap ProgowanieZprzedzialem(this Bitmap sourceBitmap, int threshold, int threshold2)
        {
            #region PROGOWANIE Z DWOMA PRZEDZIAŁAMI
            BitmapData sourceData = sourceBitmap.LockBits(new Rectangle(0, 0, sourceBitmap.Width, sourceBitmap.Height), ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
            byte[] pixelBuffer = new byte[sourceData.Stride * sourceData.Height];
            Marshal.Copy(sourceData.Scan0, pixelBuffer, 0, pixelBuffer.Length);
            sourceBitmap.UnlockBits(sourceData);

            float rgb = 0;


            for (int k = 0; k + 4 < pixelBuffer.Length; k += 4)
            {
                rgb = pixelBuffer[k] * 0.11f;
                rgb += pixelBuffer[k + 1] * 0.59f;
                rgb += pixelBuffer[k + 2] * 0.3f; 
                
                if (threshold < threshold2)
                {
                    if (pixelBuffer[k] < threshold)
                    {
                        rgb = 0;
                    }
                    if (pixelBuffer[k + 1] < threshold)
                    {
                        rgb = 0;
                    }
                    if (pixelBuffer[k + 2] < threshold)
                    {
                        rgb = 0;
                    }
                    else if (pixelBuffer[k] >= threshold2)
                    {
                        rgb = 0;
                    }
                    else if (pixelBuffer[k + 1] >= threshold2)
                    {
                       rgb = 0;
                    }
                   else if (pixelBuffer[k + 2] >= threshold2)
                    {
                        rgb = 0;
                    }
                    else if (threshold != threshold2)
                    {
                        rgb  = 255;
                    }
                }

                if (threshold2 < threshold)
                {
                    if (pixelBuffer[k] < threshold2)
                    {
                        rgb = 0;
                    }
                    if (pixelBuffer[k + 1] < threshold2)
                    {
                        rgb = 0;
                    }
                    if (pixelBuffer[k + 2] < threshold2)
                    {
                       rgb = 0;
                    }
                    else if (pixelBuffer[k] >= threshold)
                    {
                        rgb = 0;
                    }
                    else if (pixelBuffer[k + 1] >= threshold)
                    {
                        rgb = 0;
                    }
                   else if (pixelBuffer[k + 2] >= threshold)
                    {
                        rgb= 0;
                    }
                    else if (threshold != threshold2)
                    {
                       rgb  = 255;
                    }
                }

                if (rgb > 255)
                { rgb = 255; }
                else if (rgb < 0)
                { rgb = 0; }

                pixelBuffer[k] = (byte)rgb;
                pixelBuffer[k + 1] = pixelBuffer[k];
                pixelBuffer[k + 2] = pixelBuffer[k];
                pixelBuffer[k + 3] = 255;

          //      pixelBuffer[k] = (byte)blue;
           //     pixelBuffer[k + 1] = (byte)green;
          //      pixelBuffer[k + 2] = (byte)red;

            }
            Bitmap resultBitmap = new Bitmap(sourceBitmap.Width, sourceBitmap.Height);
            BitmapData resultData = resultBitmap.LockBits(new Rectangle(0, 0, resultBitmap.Width, resultBitmap.Height), ImageLockMode.WriteOnly, PixelFormat.Format32bppArgb);
            Marshal.Copy(pixelBuffer, 0, resultData.Scan0, pixelBuffer.Length);
            resultBitmap.UnlockBits(resultData);
            return resultBitmap;
            #endregion
        }

        public static Bitmap ProgowanieZprzedzialemK(this Bitmap sourceBitmap,Color darkColor, Color lightColor, int threshold, int threshold2)
        {
            #region PROGOWANIE Z DWOMA PRZEDZIAŁAMI kolor
            BitmapData sourceData = sourceBitmap.LockBits(new Rectangle(0, 0, sourceBitmap.Width, sourceBitmap.Height), ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
            byte[] pixelBuffer = new byte[sourceData.Stride * sourceData.Height];
            Marshal.Copy(sourceData.Scan0, pixelBuffer, 0, pixelBuffer.Length);
            sourceBitmap.UnlockBits(sourceData);

            float rgb = 0;


            for (int k = 0; k + 4 < pixelBuffer.Length; k += 4)
            {
                rgb = pixelBuffer[k] * 0.11f;
                rgb += pixelBuffer[k + 1] * 0.59f;
                rgb += pixelBuffer[k + 2] * 0.3f;

                if (threshold < threshold2)
                {
                    if (pixelBuffer[k] < threshold)
                    {
                        rgb = darkColor.B;
                    }
                    if (pixelBuffer[k + 1] < threshold)
                    {
                        rgb = darkColor.G;
                    }
                    if (pixelBuffer[k + 2] < threshold)
                    {
                        rgb = darkColor.R;
                    }
                    if (pixelBuffer[k] >= threshold2)
                    {
                        rgb = darkColor.B;
                    }
                    if (pixelBuffer[k + 1] >= threshold2)
                    {
                        rgb = darkColor.G;
                    }
                    if (pixelBuffer[k + 2] >= threshold2)
                    {
                        rgb = darkColor.R;
                    }
                    else if (threshold != threshold2)
                    {
                        pixelBuffer[k] = lightColor.B;
                        pixelBuffer[k + 1] = lightColor.G;
                        pixelBuffer[k + 2] = lightColor.R;
                    }
                }

                if (threshold2 < threshold)
                {
                    if (pixelBuffer[k] < threshold2)
                    {
                        rgb = darkColor.B;
                    }
                    if (pixelBuffer[k + 1] < threshold2)
                    {
                        rgb = darkColor.G; 
                    }
                    if (pixelBuffer[k + 2] < threshold2)
                    {
                        rgb = darkColor.R;
                    }
                    else if (pixelBuffer[k] >= threshold)
                    {
                        rgb = darkColor.B;
                    }
                    else if (pixelBuffer[k + 1] >= threshold)
                    {
                        rgb = darkColor.G;
                    }
                    else if (pixelBuffer[k + 2] >= threshold)
                    {
                        rgb = darkColor.R;
                    }
                    else if (threshold != threshold2)
                    {
                        pixelBuffer[k] = lightColor.B;
                        pixelBuffer[k + 1] = lightColor.G;
                        pixelBuffer[k + 2] = lightColor.R;
                    }
                }


                if (rgb > 255)
                { rgb = 255; }
                else if (rgb < 0)
                { rgb = 0; }

                pixelBuffer[k] = (byte)rgb;
               pixelBuffer[k + 1] = pixelBuffer[k];
                pixelBuffer[k + 2] = pixelBuffer[k];
                pixelBuffer[k + 3] = darkColor.R;

                //      pixelBuffer[k] = (byte)blue;
                //     pixelBuffer[k + 1] = (byte)green;
                //      pixelBuffer[k + 2] = (byte)red;

            }
            Bitmap resultBitmap = new Bitmap(sourceBitmap.Width, sourceBitmap.Height);
            BitmapData resultData = resultBitmap.LockBits(new Rectangle(0, 0, resultBitmap.Width, resultBitmap.Height), ImageLockMode.WriteOnly, PixelFormat.Format32bppArgb);
            Marshal.Copy(pixelBuffer, 0, resultData.Scan0, pixelBuffer.Length);
            resultBitmap.UnlockBits(resultData);
            return resultBitmap;
            #endregion
        }

        public static Bitmap ProgowanieZprzedzOdwrotne(this Bitmap sourceBitmap, int threshold, int threshold2)
        {
            #region PROGOWANIE Z DWOMA PRZEDZIAŁAMI ODWROTNE
            BitmapData sourceData = sourceBitmap.LockBits(new Rectangle(0, 0, sourceBitmap.Width, sourceBitmap.Height), ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
            byte[] pixelBuffer = new byte[sourceData.Stride * sourceData.Height];
            Marshal.Copy(sourceData.Scan0, pixelBuffer, 0, pixelBuffer.Length);
            sourceBitmap.UnlockBits(sourceData);

            float rgb = 0;


            for (int k = 0; k + 4 < pixelBuffer.Length; k += 4)
            {
                rgb = pixelBuffer[k] * 0.11f;
                rgb += pixelBuffer[k + 1] * 0.59f;
                rgb += pixelBuffer[k + 2] * 0.3f;

                if (threshold < threshold2)
                {
                    if (pixelBuffer[k] < threshold)
                    {
                        rgb = 255;
                    }
                    if (pixelBuffer[k + 1] < threshold)
                    {
                        rgb = 255;
                    }
                    if (pixelBuffer[k + 2] < threshold)
                    {
                        rgb = 255;
                    }
                    else if (pixelBuffer[k] >= threshold2)
                    {
                        rgb = 255;
                    }
                    else if (pixelBuffer[k + 1] >= threshold2)
                    {
                        rgb = 255;
                    }
                    else if (pixelBuffer[k + 2] >= threshold2)
                    {
                        rgb = 255;
                    }
                    else if (threshold != threshold2)
                    {
                        rgb = 0;
                    }
                }

                if (threshold2 < threshold)
                {
                    if (pixelBuffer[k] < threshold2)
                    {
                        rgb = 255;
                    }
                    if (pixelBuffer[k + 1] < threshold2)
                    {
                        rgb = 255;
                    }
                    if (pixelBuffer[k + 2] < threshold2)
                    {
                        rgb = 255;
                    }
                    else if (pixelBuffer[k] >= threshold)
                    {
                        rgb = 255;
                    }
                    else if (pixelBuffer[k + 1] >= threshold)
                    {
                        rgb = 255;
                    }
                    else if (pixelBuffer[k + 2] >= threshold)
                    {
                        rgb = 255;
                    }
                    else if (threshold != threshold2)
                    {
                        rgb = 0;
                    }
                }

                if (rgb > 255)
                { rgb = 255; }
                else if (rgb < 0)
                { rgb = 0; }

                pixelBuffer[k] = (byte)rgb;
                pixelBuffer[k + 1] = pixelBuffer[k];
                pixelBuffer[k + 2] = pixelBuffer[k];
                pixelBuffer[k + 3] = 255;

                //      pixelBuffer[k] = (byte)blue;
                //     pixelBuffer[k + 1] = (byte)green;
                //      pixelBuffer[k + 2] = (byte)red;

            }
            Bitmap resultBitmap = new Bitmap(sourceBitmap.Width, sourceBitmap.Height);
            BitmapData resultData = resultBitmap.LockBits(new Rectangle(0, 0, resultBitmap.Width, resultBitmap.Height), ImageLockMode.WriteOnly, PixelFormat.Format32bppArgb);
            Marshal.Copy(pixelBuffer, 0, resultData.Scan0, pixelBuffer.Length);
            resultBitmap.UnlockBits(resultData);
            return resultBitmap;
            #endregion
        }

        public static Bitmap ProgowanieZprzedzOdwrotneKolor2(this Bitmap sourceBitmap, Color darkColor, Color lightColor, int threshold, int threshold2)
        {
            #region PROGOWANIE Z DWOMA PRZEDZIAŁAMI ODWROTNE kolor
            BitmapData sourceData = sourceBitmap.LockBits(new Rectangle(0, 0, sourceBitmap.Width, sourceBitmap.Height), ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
            byte[] pixelBuffer = new byte[sourceData.Stride * sourceData.Height];
            Marshal.Copy(sourceData.Scan0, pixelBuffer, 0, pixelBuffer.Length);
            sourceBitmap.UnlockBits(sourceData);

          //  float rgb = 0;


            for (int k = 0; k + 4 < pixelBuffer.Length; k += 4)
            {
          //      rgb = pixelBuffer[k] * 0.11f;
         //       rgb += pixelBuffer[k + 1] * 0.59f;
         //       rgb += pixelBuffer[k + 2] * 0.3f;

                if (threshold < threshold2)
                {
                    if (pixelBuffer[k] < threshold)
                    {
                       pixelBuffer[k] = darkColor.B;
                    }
                    if (pixelBuffer[k + 1] < threshold)
                    {
                       pixelBuffer[k + 1] = darkColor.G;
                    }
                    if (pixelBuffer[k + 2] < threshold)
                    {
                        pixelBuffer[k + 2] = darkColor.R;
                    }
                    else if (pixelBuffer[k] >= threshold2)
                    {
                       pixelBuffer[k] = darkColor.B;
                    }
                    else if (pixelBuffer[k + 1] >= threshold2)
                    {
                       pixelBuffer[k + 1] = darkColor.G;
                    }
                    else if (pixelBuffer[k + 2] >= threshold2)
                    {
                       pixelBuffer[k + 2] = darkColor.R;
                    }
                    else if (threshold != threshold2)
                    {
                        pixelBuffer[k] = lightColor.B;
                        pixelBuffer[k + 1] = lightColor.G;
                        pixelBuffer[k + 2] = lightColor.R;
                    }
                }

                if (threshold2 < threshold)
                {
                    if (pixelBuffer[k] < threshold2)
                    {
                       pixelBuffer[k] = darkColor.B;
                    }
                    if (pixelBuffer[k + 1] < threshold2)
                    {
                       pixelBuffer[k + 1] = darkColor.G;
                    }
                    if (pixelBuffer[k + 2] < threshold2)
                    {
                        pixelBuffer[k + 2] = darkColor.R;
                    }
                    else if (pixelBuffer[k] >= threshold)
                    {
                        pixelBuffer[k] = darkColor.B;
                    }
                    else if (pixelBuffer[k + 1] >= threshold)
                    {
                        pixelBuffer[k + 1] = darkColor.G;
                    }
                    else if (pixelBuffer[k + 2] >= threshold)
                    {
                        pixelBuffer[k + 2] = darkColor.R;
                    }
                    else if (threshold != threshold2)
                    {
                        pixelBuffer[k] = lightColor.B;
                        pixelBuffer[k + 1] = lightColor.G;
                        pixelBuffer[k + 2] = lightColor.R;
                    }
                }

                if ( pixelBuffer[k] > 255)
                {  pixelBuffer[k]= 255; }
                else if (pixelBuffer[k] < 0)
                { pixelBuffer[k] = 0; }

                if ( pixelBuffer[k+1] > 255)
                {  pixelBuffer[k+1]= 255; }
                else if ( pixelBuffer[k+1] < 0)
                {  pixelBuffer[k+1]= 0; }

                if ( pixelBuffer[k+2] > 255)
                {  pixelBuffer[k+2]= 255; }
                else if ( pixelBuffer[k+2] < 0)
                {  pixelBuffer[k+2]= 0; }

                 //      pixelBuffer[k] = (byte)blue;
                //     pixelBuffer[k + 1] = (byte)green;
                //      pixelBuffer[k + 2] = (byte)red;
                }
            Bitmap resultBitmap = new Bitmap(sourceBitmap.Width, sourceBitmap.Height);
            BitmapData resultData = resultBitmap.LockBits(new Rectangle(0, 0, resultBitmap.Width, resultBitmap.Height), ImageLockMode.WriteOnly, PixelFormat.Format32bppArgb);
            Marshal.Copy(pixelBuffer, 0, resultData.Scan0, pixelBuffer.Length);
            resultBitmap.UnlockBits(resultData);
            return resultBitmap;
            #endregion
        }

        public static Bitmap RedukcjaPoziomowSzarosci(this Bitmap sourceBitmap, int threshold)
        {
            #region REDUKCJA POZIOMÓW SZAROŚCI
            BitmapData sourceData = sourceBitmap.LockBits(new Rectangle(0, 0, sourceBitmap.Width, sourceBitmap.Height), ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
            byte[] pixelBuffer = new byte[sourceData.Stride * sourceData.Height];
            Marshal.Copy(sourceData.Scan0, pixelBuffer, 0, pixelBuffer.Length);
            sourceBitmap.UnlockBits(sourceData);

            float krok = 255 / threshold;

//            float blue = 0;
  //          float green = 0;
    //        float red = 0;

            float blue1 = 0;
            float green1 = 0;
            float red1 = 0;

             for (int k = 0; k + 4 < pixelBuffer.Length; k += 4)
            {
                blue1 = pixelBuffer[k] / krok;
                green1 = pixelBuffer[k + 1] / krok;
                red1 = pixelBuffer[k + 2] /krok;
                
                 if (blue1 == threshold  || blue1 == threshold-1)
                 {
                     blue1 = 255;              
                 }
                 if (green1 == threshold  || green1 == threshold-1)
                 {
                     green1 = 255;              
                 }
                 if (red1 == threshold  ||red1 == threshold-1)
                 {
                     red1 = 255;              
                 }
                 if (blue1 == 0)
                 {
                    blue1 = 0;
                 }
                 if (green1 == 0)
                 {
                    green1 = 0;
                 }
                 if (red1 == 0)
                 {
                     red1 = 0;
                 }
                 else 
                 { 
                    blue1  =  (pixelBuffer[k] + krok/2);
                    green1 =  (pixelBuffer[k+1]  + krok/2);
                    red1 =    (pixelBuffer[k+2]  + krok/2);           
                 }

                if (blue1 > 255)
                { blue1 = 255; }

                if (green1 > 255)
                { green1 = 255; }

                if (red1 > 255)
                { red1 = 255; }

                pixelBuffer[k] = (byte)blue1;
                pixelBuffer[k + 1] = (byte)green1;
                pixelBuffer[k + 2] = (byte)red1;
            }         
                
            Bitmap resultBitmap = new Bitmap(sourceBitmap.Width, sourceBitmap.Height);
            BitmapData resultData = resultBitmap.LockBits(new Rectangle(0, 0, resultBitmap.Width, resultBitmap.Height), ImageLockMode.WriteOnly, PixelFormat.Format32bppArgb);
            Marshal.Copy(pixelBuffer, 0, resultData.Scan0, pixelBuffer.Length);
            resultBitmap.UnlockBits(resultData);
            return resultBitmap;
            #endregion
        }


/*
        
         //  Bitmap bmp1 = new Bitmap(bmp);
            BitmapProcessing.FastBitmap fobrazek_bitmapa = new BitmapProcessing.FastBitmap(bmp);
         //   fobrazek_bitmapa.LockImage();

            for (int x = 0; x < bmp.Width; x++)
            {
                for (int y = 0; y < bmp.Height; y++)
                {
                    int wartosc_piksela = bmp.GetPixel(x, y).R;

                    int przedzial = (int)(wartosc_piksela / war1);

                    if (przedzial == war1 || przedzial == war1 - 1)
                    {
                        bmp.SetPixel(x, y, Color.FromArgb(255, 255, 255));
                    }
                    else if (przedzial == 0)
                    {
                        bmp.SetPixel(x, y, Color.FromArgb(0, 0, 0));
                    }
                    else
                    {
                        int tmp = (int)(((float)przedzial) * krok + krok / 2);
                        bmp.SetPixel(x, y, Color.FromArgb(tmp, tmp, tmp));
                    }
                }
*/


        /*
        public static Bitmap ProgowanieOdwrotne(this Bitmap sourceBitmap, int threshold)
        {
            #region Progowanie odwrotne
            BitmapData sourceData = sourceBitmap.LockBits(new Rectangle(0, 0, sourceBitmap.Width, sourceBitmap.Height), ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
            byte[] pixelBuffer = new byte[sourceData.Stride * sourceData.Height];
            Marshal.Copy(sourceData.Scan0, pixelBuffer, 0, pixelBuffer.Length);
            sourceBitmap.UnlockBits(sourceData);

            double blue = 0;
            double green = 0;
            double red = 0;

            for (int k = 0; k + 4 < pixelBuffer.Length; k += 4)
            {
                if (pixelBuffer[k] <= threshold)
                {
                    blue = pixelBuffer[k] = 255;
                }
                else blue = pixelBuffer[k] = 0;

                if (pixelBuffer[k + 1] <= threshold)
                {
                    green = pixelBuffer[k + 1] = 255;
                }
                else green = pixelBuffer[k + 1] = 0;

                if (pixelBuffer[k + 2] <= threshold)
                {
                    red = pixelBuffer[k + 2] = 255;
                }
                else red = pixelBuffer[k + 2] = 0;


                if (blue > 255)
                { blue = 255; }
                else if (blue < 0)
                { blue = 0; }

                if (green > 255)
                { green = 255; }
                else if (green < 0)
                { green = 0; }

                if (red > 255)
                { red = 255; }
                else if (red < 0)
                { red = 0; }

                pixelBuffer[k] = (byte)blue;
                pixelBuffer[k + 1] = (byte)green;
                pixelBuffer[k + 2] = (byte)red;
            }
            Bitmap resultBitmap = new Bitmap(sourceBitmap.Width, sourceBitmap.Height);
            BitmapData resultData = resultBitmap.LockBits(new Rectangle(0, 0, resultBitmap.Width, resultBitmap.Height), ImageLockMode.WriteOnly, PixelFormat.Format32bppArgb);
            Marshal.Copy(pixelBuffer, 0, resultData.Scan0, pixelBuffer.Length);
            resultBitmap.UnlockBits(resultData);
            return resultBitmap;
            #endregion
        }
         */

        public static Bitmap Progowanie2(this Bitmap sourceBitmap, int threshold, int threshold2)
        {
            #region Progowanie z zachoowaniem poziomów szarości
            BitmapData sourceData = sourceBitmap.LockBits(new Rectangle(0, 0, sourceBitmap.Width, sourceBitmap.Height), ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
            byte[] pixelBuffer = new byte[sourceData.Stride * sourceData.Height];
            Marshal.Copy(sourceData.Scan0, pixelBuffer, 0, pixelBuffer.Length);
            sourceBitmap.UnlockBits(sourceData);

            double rgb = 0;

            for (int k = 0; k + 4 < pixelBuffer.Length; k += 4)
            {
                rgb = pixelBuffer[k] * 0.11f;
                rgb += pixelBuffer[k + 1] * 0.59f;
                rgb += pixelBuffer[k + 2] * 0.3f;


                if (threshold < threshold2)
                {
                    if (pixelBuffer[k] <= threshold || pixelBuffer[k] > threshold2)
                    {
                        rgb = pixelBuffer[k] = 0;
                    }
                    else rgb = pixelBuffer[k] + threshold * (pixelBuffer.Length / 256.0);

                    if (pixelBuffer[k + 1] <= threshold || pixelBuffer[k + 1] > threshold2)
                    {
                        rgb = pixelBuffer[k + 1] = 0;
                    }
                    else rgb = pixelBuffer[k + 1] + threshold * (pixelBuffer.Length / 256.0);

                    if (pixelBuffer[k + 2] <= threshold || pixelBuffer[k + 2] > threshold2)
                    {
                        rgb = pixelBuffer[k + 2] = 0;
                    }
                    else rgb = pixelBuffer[k + 2] + threshold * (pixelBuffer.Length / 256.0);
                }
                else 
                {
                    if (pixelBuffer[k] <= threshold2 || pixelBuffer[k] > threshold)
                    {
                        rgb = pixelBuffer[k] = 0;
                    }
                    else rgb = pixelBuffer[k] + threshold2 * (pixelBuffer.Length / 256.0);

                    if (pixelBuffer[k + 1] <= threshold2 || pixelBuffer[k + 1] > threshold)
                    {
                        rgb = pixelBuffer[k + 1] = 0;
                    }
                    else rgb = pixelBuffer[k + 1] + threshold2 * (pixelBuffer.Length / 256.0);

                    if (pixelBuffer[k + 2] <= threshold2 || pixelBuffer[k + 2] > threshold)
                    {
                        rgb = pixelBuffer[k + 2] = 0;
                    }
                    else rgb = pixelBuffer[k + 2] + threshold2 * (pixelBuffer.Length / 256.0);                            
                }

                if (rgb > 255)
                { rgb = 255; }
                else if (rgb < 0)
                { rgb = 0; }

                pixelBuffer[k] = (byte)rgb;
                pixelBuffer[k + 1] = pixelBuffer[k];
                pixelBuffer[k + 2] = pixelBuffer[k];
            }
            Bitmap resultBitmap = new Bitmap(sourceBitmap.Width, sourceBitmap.Height);
            BitmapData resultData = resultBitmap.LockBits(new Rectangle(0, 0, resultBitmap.Width, resultBitmap.Height), ImageLockMode.WriteOnly, PixelFormat.Format32bppArgb);
            Marshal.Copy(pixelBuffer, 0, resultData.Scan0, pixelBuffer.Length);
            resultBitmap.UnlockBits(resultData);
            return resultBitmap;
            #endregion
        }

        public static Bitmap Progowanie2Odwrot(this Bitmap sourceBitmap, int threshold, int threshold2)
        {
            #region Progowanie z zachowaniem poziomów szarości odwrotne
            BitmapData sourceData = sourceBitmap.LockBits(new Rectangle(0, 0, sourceBitmap.Width, sourceBitmap.Height), ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
            byte[] pixelBuffer = new byte[sourceData.Stride * sourceData.Height];
            Marshal.Copy(sourceData.Scan0, pixelBuffer, 0, pixelBuffer.Length);
            sourceBitmap.UnlockBits(sourceData);

            double rgb = 0;

            for (int k = 0; k + 4 < pixelBuffer.Length; k += 4)
            {
                rgb = pixelBuffer[k] * 0.11f;
                rgb += pixelBuffer[k + 1] * 0.59f;
                rgb += pixelBuffer[k + 2] * 0.3f;


                if (threshold < threshold2)
                {
                    if (pixelBuffer[k] < threshold || pixelBuffer[k] > threshold2)
                    {
                        rgb = pixelBuffer[k] = 0;
                    }
                    else rgb = (pixelBuffer[k] + threshold * (pixelBuffer.Length / 256.0));

                    if (pixelBuffer[k + 1] < threshold || pixelBuffer[k + 1] > threshold2)
                    {
                        rgb = pixelBuffer[k + 1] = 0;
                    }
                    else rgb = (pixelBuffer[k + 1] + threshold * (pixelBuffer.Length / 256.0));

                    if (pixelBuffer[k + 2] < threshold || pixelBuffer[k + 2] > threshold2)
                    {
                        rgb = pixelBuffer[k + 2] = 0;
                    }
                    else rgb = (pixelBuffer[k + 2] + threshold * ( pixelBuffer.Length / 256.0));
                }
                else
                {
                    if (pixelBuffer[k] < threshold2 || pixelBuffer[k] > threshold)
                    {
                        rgb = pixelBuffer[k] = 0;
                    }
                    else rgb = (pixelBuffer[k] + threshold2 * (pixelBuffer.Length / 256.0));

                    if (pixelBuffer[k + 1] < threshold2 || pixelBuffer[k + 1] > threshold)
                    {
                        rgb = pixelBuffer[k + 1] = 0;
                    }
                    else rgb =  (pixelBuffer[k + 1] + threshold2 * ( pixelBuffer.Length / 256.0));

                    if (pixelBuffer[k + 2] < threshold2 || pixelBuffer[k + 2] > threshold)
                    {
                        rgb = pixelBuffer[k + 2] = 0;
                    }
                    else rgb = (pixelBuffer[k + 2] + threshold2 * (pixelBuffer.Length / 256.0));
                }

                if (rgb > 255)
                { rgb = 255; }
                else if (rgb < 0)
                { rgb = 0; }

                pixelBuffer[k] =  (byte)(255 - rgb);
                pixelBuffer[k + 1] =( pixelBuffer[k]);
                pixelBuffer[k + 2] = (pixelBuffer[k]);
            }
            Bitmap resultBitmap = new Bitmap(sourceBitmap.Width, sourceBitmap.Height);
            BitmapData resultData = resultBitmap.LockBits(new Rectangle(0, 0, resultBitmap.Width, resultBitmap.Height), ImageLockMode.WriteOnly, PixelFormat.Format32bppArgb);
            Marshal.Copy(pixelBuffer, 0, resultData.Scan0, pixelBuffer.Length);
            resultBitmap.UnlockBits(resultData);
            return resultBitmap;
            #endregion
        }

        public static Bitmap PiramW(this Bitmap sourceBitmap, int w)
        {
            #region Piramida algorytm
            Bitmap bmpHSV = new Bitmap(sourceBitmap);
            BitmapData data = bmpHSV.LockBits(new System.Drawing.Rectangle(0, 0, bmpHSV.Width, bmpHSV.Height), ImageLockMode.ReadWrite, PixelFormat.Format32bppArgb);

            int negR, negG, negB;
            if (w == 0)
            {
                w = 1;
            }

            for (int x = 0; x < sourceBitmap.Width; x++)
                for (int y = 0; y < sourceBitmap.Height; y++)
                {
                    Color col = sourceBitmap.GetPixel(x, y);
                    negR = col.R;
                    negG = col.G;
                    negB = col.B;
                    Color col1 = Color.FromArgb(col.A, negR, negG, negB);
                    sourceBitmap.SetPixel(x, y, col1);
                }
            bmpHSV.UnlockBits(data);
            sourceBitmap = new Bitmap(sourceBitmap, sourceBitmap.Width / w, sourceBitmap.Height / w);
            return sourceBitmap;
            #endregion
        }       

        public static Bitmap AlgorytmZolw(this Bitmap sourceBitmap)
        {
            #region AlgorytmZolw
            BitmapData sourceData = sourceBitmap.LockBits(new Rectangle(0, 0, sourceBitmap.Width, sourceBitmap.Height), ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
            byte[] pixelBuffer = new byte[sourceData.Stride * sourceData.Height];
            Marshal.Copy(sourceData.Scan0, pixelBuffer, 0, pixelBuffer.Length);
            sourceBitmap.UnlockBits(sourceData);

            int[,] rtab = new int[sourceBitmap.Width, sourceBitmap.Height];
            int[,] gtab = new int[sourceBitmap.Width, sourceBitmap.Height];
            int[,] btab = new int[sourceBitmap.Width, sourceBitmap.Height];
            int i, j;
            for (i = 1; i < sourceBitmap.Width - 1; i++)
            {
                for (j = 1; j < sourceBitmap.Height - 1; j++)
                {
                    rtab[i, j] = sourceBitmap.GetPixel(i, j).R;
                    gtab[i, j] = sourceBitmap.GetPixel(i, j).G;
                    btab[i, j] = sourceBitmap.GetPixel(i, j).B;
                }
            }
            int d = 0;
            int pami = 0, pamj = 0, ja = 0, ia = 0;
            int x, y;
            int[] wynik = new int[sourceBitmap.Width * sourceBitmap.Height];
            int[] droga = new int[sourceBitmap.Width * sourceBitmap.Height];
            for (i = 1; i < sourceBitmap.Height - 1; i++)
            {
                for (j = 1; j < sourceBitmap.Width - 1; j++)
                {
                    if (rtab[j, i] != 0 || gtab[j, i] != 0 || btab[j, i] != 0)
                    {
                        ja = j;
                        ia = i;
                        pamj = j;
                        pami = i;
                        wynik[sourceBitmap.Width * i + j] = 255;
                        goto cont;
                    }
                }
            }
        cont:
            j = pamj;
            i = pami - 1;
            wynik[sourceBitmap.Width * i + j] = 255;
            droga[d] = 1;
            do
            {
                x = j - pamj;
                y = i - pami;
                pamj = j;
                pami = i;
                d++;
                if (rtab[j, i] != 0 || gtab[j, i] != 0 || btab[j, i] != 0)
                {
                    if (x == 0 && y == (-1))
                    {
                        j--;
                        droga[d] = 2;
                    }
                    if (x == 1 && y == 0)
                    {
                        i--;
                        droga[d] = 1;
                    }
                    if (x == 0 && y == 1)
                    {
                        j++;
                        droga[d] = 0;
                    }
                    if (x == (-1) && y == 0)
                    {
                        i++;
                        droga[d] = 3;
                    }
                }
                else
                {
                    if (x == 0 && y == (-1))
                    {
                        j++;
                        droga[d] = 0;
                    }
                    if (x == 1 && y == 0)
                    {
                        i++;
                        droga[d] = 3;
                    }
                    if (x == 0 && y == 1)
                    {
                        j--;
                        droga[d] = 2;
                    }
                    if (x == (-1) && y == 0)
                    {
                        i--;
                        droga[d] = 1;
                    }
                }
                wynik[sourceBitmap.Width * i + j] = 255;
            }
            while (j != ja || i != ia);
            for (i = 0; i < sourceBitmap.Height; i++)
            {
                for (j = 0; j < sourceBitmap.Width; j++)
                {
                    if (wynik[sourceBitmap.Width * i + j] == 255)
                    {
                        rtab[j, i] = 255;
                        gtab[j, i] = 0;
                        btab[j, i] = 0;
                    }
                }
            }

            for (i = 0; i < sourceBitmap.Width; i++)
            {
                for (j = 0; j < sourceBitmap.Height; j++)
                {
                    sourceBitmap.SetPixel(i, j, Color.FromArgb(rtab[i, j], gtab[i, j], btab[i, j]));
                }
            }
            return sourceBitmap;
            #endregion
        }

        public static Bitmap AlgorytmZolw2(this Bitmap sourceBitmap, int startx, int starty)
        {
            #region AlgorytmZolw2
            BitmapData sourceData = sourceBitmap.LockBits(new Rectangle(0, 0, sourceBitmap.Width, sourceBitmap.Height), ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
            byte[] pixelBuffer = new byte[sourceData.Stride * sourceData.Height];
            Marshal.Copy(sourceData.Scan0, pixelBuffer, 0, pixelBuffer.Length);
            sourceBitmap.UnlockBits(sourceData);

            int[] dx = { -1, 0, 1, 0 };
            int[] dy = { 0, -1, 0, 1 };
            int x = -1; int y = -1;

            if (sourceBitmap.GetPixel(startx, starty).B >= 100)
            {
                for (int xx = 0; xx < sourceBitmap.Width; ++xx)
                {
                    for (int yy = 0; yy < sourceBitmap.Height; ++yy)
                    {
                        if (sourceBitmap.GetPixel(xx, yy).B < 100) { x = xx; y = yy; }
                    }
                }
            }
            else
            {
                for (int xx = startx; xx < sourceBitmap.Width; ++xx)
                {
                    y = starty;
                    x = sourceBitmap.Width - 1;
                    if (sourceBitmap.GetPixel(xx, starty).B >= 100)
                    {
                        x = xx - 1;
                        break;
                    }
                }
            }
            int d = 0;
            startx = x;
            starty = y;
            Bitmap ret = new Bitmap(sourceBitmap);
            do
            {
                if (x >= 0 && y >= 0 && x < sourceBitmap.Width && y < sourceBitmap.Height)
                    ret.SetPixel(x, y, Color.FromArgb(255, 0, 0));

                if (x >= 0 && y >= 0 && x < sourceBitmap.Width && y < sourceBitmap.Height &&
                    sourceBitmap.GetPixel(x, y).B < 100)
                {
                    d = (d + 3) % 4;
                }
                else
                {
                    d = (d + 1) % 4;
                }
                x += dx[d];
                y += dy[d];
            } while (x != startx || y != starty);
            return ret;
            #endregion
        }

        public static Bitmap ModelRGB(this Bitmap sourceBitmap)
        {
            #region ModelRGB
            BitmapData sourceData = sourceBitmap.LockBits(new Rectangle(0, 0, sourceBitmap.Width, sourceBitmap.Height), ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
            byte[] pixelBuffer = new byte[sourceData.Stride * sourceData.Height];
            Marshal.Copy(sourceData.Scan0, pixelBuffer, 0, pixelBuffer.Length);
            sourceBitmap.UnlockBits(sourceData);

            double blue = 0;
            double green = 0;
            double red = 0;

            for (int k = 0; k + 4 < pixelBuffer.Length; k += 4)
            {
               blue = pixelBuffer[k];
               green = pixelBuffer[k + 1];
               red = pixelBuffer[k + 2];

                if (blue > 255)
                { blue = 255; }
                else if (blue < 0)
                { blue = 0; }

                if (green > 255)
                { green = 255; }
                else if (green < 0)
                { green = 0; }

                if (red > 255)
                { red = 255; }
                else if (red < 0)
                { red = 0; }

                pixelBuffer[k] = (byte)blue;
                pixelBuffer[k+1] = (byte)green;
                pixelBuffer[k+2] = (byte)red;
            }
            Bitmap resultBitmap = new Bitmap(sourceBitmap.Width, sourceBitmap.Height);
            BitmapData resultData = resultBitmap.LockBits(new Rectangle(0, 0, resultBitmap.Width, resultBitmap.Height), ImageLockMode.WriteOnly, PixelFormat.Format32bppArgb);
            Marshal.Copy(pixelBuffer, 0, resultData.Scan0, pixelBuffer.Length);
            resultBitmap.UnlockBits(resultData);
            return resultBitmap;
            #endregion
        }

           public static Bitmap ModelRGB_RedNew(this Bitmap sourceBitmap)
           {
               BitmapData sourceData = sourceBitmap.LockBits(new Rectangle(0, 0, sourceBitmap.Width, sourceBitmap.Height), ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
               byte[] pixelBuffer = new byte[sourceData.Stride * sourceData.Height];
               Marshal.Copy(sourceData.Scan0, pixelBuffer, 0, pixelBuffer.Length);
               sourceBitmap.UnlockBits(sourceData);

               double blue = 0;
               double green = 0;
               double red = 0;

               for (int k = 0; k + 4 < pixelBuffer.Length; k += 4)
               {
                  blue = pixelBuffer[k] = pixelBuffer[k + 1] = pixelBuffer[k + 2];
                  green = pixelBuffer[k] = pixelBuffer[k + 1] = pixelBuffer[k + 2];
                  red = pixelBuffer[k] = pixelBuffer[k + 1] = pixelBuffer[k + 2];

                   //  blue = pixelBuffer[0] = pixelBuffer[1] = pixelBuffer[2];
                   blue += 3;
                   green += 3;
                   red += 3;

                   if (blue > 255)
                   { blue = 255; }
                   else if (blue < 0)
                   { blue = 0; }

                   if (green > 255)
                   { green = 255; }
                   else if (green < 0)
                   { green = 0; }

                   if (red > 255)
                   { red = 255; }
                   else if (red < 0)
                   { red = 0; }

                   pixelBuffer[k] = (byte)red;
                   pixelBuffer[k + 1] = (byte)red;
                   pixelBuffer[k + 2] = (byte)red;

               //    pixelBuffer[k + 1] = (byte)green;
               //    pixelBuffer[k + 2] = (byte)red;
               }
               Bitmap resultBitmap = new Bitmap(sourceBitmap.Width, sourceBitmap.Height);
               BitmapData resultData = resultBitmap.LockBits(new Rectangle(0, 0, resultBitmap.Width, resultBitmap.Height), ImageLockMode.WriteOnly, PixelFormat.Format32bppArgb);
               Marshal.Copy(pixelBuffer, 0, resultData.Scan0, pixelBuffer.Length);
               resultBitmap.UnlockBits(resultData);
               return resultBitmap;
           }

           public static Bitmap ModelRGB_GreenNew(this Bitmap sourceBitmap)
           {
               BitmapData sourceData = sourceBitmap.LockBits(new Rectangle(0, 0, sourceBitmap.Width, sourceBitmap.Height), ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
               byte[] pixelBuffer = new byte[sourceData.Stride * sourceData.Height];
               Marshal.Copy(sourceData.Scan0, pixelBuffer, 0, pixelBuffer.Length);
               sourceBitmap.UnlockBits(sourceData);

               double blue = 0;
               double green = 0;
               double red = 0;

               for (int k = 0; k + 4 < pixelBuffer.Length; k += 4)
               {
                   blue = pixelBuffer[k] = pixelBuffer[k + 1] = pixelBuffer[k + 2];
                   green = pixelBuffer[k] = pixelBuffer[k + 1] = pixelBuffer[k + 2];
                   red = pixelBuffer[k] = pixelBuffer[k + 1] = pixelBuffer[k + 2];

                   //  blue = pixelBuffer[0] = pixelBuffer[1] = pixelBuffer[2];
                   blue += 3;
                   green += 3;
                   red += 3;

                   if (blue > 255)
                   { blue = 255; }
                   else if (blue < 0)
                   { blue = 0; }

                   if (green > 255)
                   { green = 255; }
                   else if (green < 0)
                   { green = 0; }

                   if (red > 255)
                   { red = 255; }
                   else if (red < 0)
                   { red = 0; }

                   pixelBuffer[k] = (byte)green;
                   pixelBuffer[k + 1] = (byte)green;
                   pixelBuffer[k + 2] = (byte)green;

                   //    pixelBuffer[k + 1] = (byte)green;
                   //    pixelBuffer[k + 2] = (byte)red;
               }
               Bitmap resultBitmap = new Bitmap(sourceBitmap.Width, sourceBitmap.Height);
               BitmapData resultData = resultBitmap.LockBits(new Rectangle(0, 0, resultBitmap.Width, resultBitmap.Height), ImageLockMode.WriteOnly, PixelFormat.Format32bppArgb);
               Marshal.Copy(pixelBuffer, 0, resultData.Scan0, pixelBuffer.Length);
               resultBitmap.UnlockBits(resultData);
               return resultBitmap;
           }

           public static Bitmap ModelRGB_BluenNew(this Bitmap sourceBitmap)
           {
               BitmapData sourceData = sourceBitmap.LockBits(new Rectangle(0, 0, sourceBitmap.Width, sourceBitmap.Height), ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
         //      byte[] pixelBuffer = new byte[sourceData.Stride * sourceData.Height];
         //      Marshal.Copy(sourceData.Scan0, pixelBuffer, 0, pixelBuffer.Length);
        //       sourceBitmap.UnlockBits(sourceData);

            //   double blue = 0;
            //   double green = 0;
            //   double red = 0;

               IntPtr ptr = sourceData.Scan0;
               int bytes = sourceData.Stride * sourceBitmap.Height;           
               byte[] rgbValues = new byte[bytes];
               System.Runtime.InteropServices.Marshal.Copy(ptr, rgbValues, 0, bytes);
               byte[] Red = new byte[sourceBitmap.Width * sourceBitmap.Height];
               byte[] Green = new byte[sourceBitmap.Width * sourceBitmap.Height];
               byte[] Blue = new byte[sourceBitmap.Width * sourceBitmap.Height];

   
               int cnt = 0;
               for (int pos = 0; pos + 2 < rgbValues.Length; pos += 3)
               {
                   if ((cnt != 0) && ((cnt % sourceBitmap.Width) == 0))
                   {
                       pos = pos + 2;

                       for (int a = 1; a <= 0; a--)
                       {
                           Red[cnt] = rgbValues[pos];
                           Green[cnt] = rgbValues[pos];
                           Blue[cnt] = rgbValues[pos];
                           cnt++;
                           pos++;
                       }
                   }



                Red[cnt] = rgbValues[pos + 2];
                Green[cnt] = rgbValues[pos + 1];
                Blue[cnt] = rgbValues[pos];           
                cnt++;
      }
        
    //  Bitmap newbmp = new Bitmap(sourceData.Width, sourceData.Height, PixelFormat.Format24bppRgb);
   //   BitmapData newbmpData = newbmp.LockBits(new Rectangle(0, 0, newbmp.Width, newbmp.Height), ImageLockMode.WriteOnly, newbmp.PixelFormat);
   //   System.Runtime.InteropServices.Marshal.Copy(Red, 0, newbmpData.Scan0, Red.Length);
 //     sourceBitmap.UnlockBits(sourceData);
  //    newbmp.UnlockBits(newbmpData);
  //    myBitmap.Dispose();
  //    myBitmap = newbmp;    
  //    doubleBufferPanel1.Invalidate();
    
      Bitmap resultBitmap = new Bitmap(sourceBitmap.Width, sourceBitmap.Height, PixelFormat.Format24bppRgb);
      BitmapData resultData = resultBitmap.LockBits(new Rectangle(0, 0, resultBitmap.Width, resultBitmap.Height), ImageLockMode.WriteOnly, PixelFormat.Format32bppArgb);
      Marshal.Copy(Red, 0, resultData.Scan0, Red.Length);
      resultBitmap.UnlockBits(resultData);
      return resultBitmap;
      }



        //----------
           public static Bitmap ModelRGB_Red(this Bitmap sourceBitmap)
           {
               BitmapData sourceData = sourceBitmap.LockBits(new Rectangle(0, 0, sourceBitmap.Width, sourceBitmap.Height), ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
               byte[] pixelBuffer = new byte[sourceData.Stride * sourceData.Height];
               Marshal.Copy(sourceData.Scan0, pixelBuffer, 0, pixelBuffer.Length);
               sourceBitmap.UnlockBits(sourceData);
               //  double contrastLevel = (100 + threshold * 0.390625) / 100;
               //   double contrastLevel = Math.Pow((100.0 + threshold) / 100.0, 2);

             //  double blue = 0;
               //    double green = 0;
               double red = 0;


               for (int k = 0; k + 4 < pixelBuffer.Length; k += 4)
               {
                //   red = pixelBuffer[k] = pixelBuffer[k + 1] = pixelBuffer[k + 2];
                   red = pixelBuffer[0] = pixelBuffer[1] = pixelBuffer[2];
                   red += 3;

                   if (red > 255)
                   { red = 255; }
                   else if (red < 0)
                   { red = 0; }

                   pixelBuffer[k] = (byte)red;
               }
               Bitmap resultBitmap = new Bitmap(sourceBitmap.Width, sourceBitmap.Height);
               BitmapData resultData = resultBitmap.LockBits(new Rectangle(0, 0, resultBitmap.Width, resultBitmap.Height), ImageLockMode.WriteOnly, PixelFormat.Format32bppArgb);
               Marshal.Copy(pixelBuffer, 0, resultData.Scan0, pixelBuffer.Length);
               resultBitmap.UnlockBits(resultData);
               return resultBitmap;
           }
        
           public static Bitmap ObrazRGB_red(this Bitmap sourceBitmap)
           {
               #region ObrazRGB_red
               Bitmap bmpHSV1 = new Bitmap(sourceBitmap);
               BitmapData data = bmpHSV1.LockBits(new System.Drawing.Rectangle(0, 0, bmpHSV1.Width, bmpHSV1.Height), ImageLockMode.ReadWrite, PixelFormat.Format32bppArgb);
               int r;

               for (int x = 0; x < sourceBitmap.Width; x++)
                   for (int y = 0; y < sourceBitmap.Height; y++)
                   {
                       Color col = sourceBitmap.GetPixel(x, y);
                       r = col.R;
                       Color col1 = Color.FromArgb(col.A, r, r,r);
                       sourceBitmap.SetPixel(x, y, col1);
                   }
               bmpHSV1.UnlockBits(data);
               return sourceBitmap;
               #endregion
           }

           public static Bitmap ObrazRGB_green(this Bitmap sourceBitmap)
           {
               #region ObrazRGB_green
               Bitmap bmpHSV1 = new Bitmap(sourceBitmap);
               BitmapData data = bmpHSV1.LockBits(new System.Drawing.Rectangle(0, 0, bmpHSV1.Width, bmpHSV1.Height), ImageLockMode.ReadWrite, PixelFormat.Format32bppArgb);
               int g;

               for (int x = 0; x < sourceBitmap.Width; x++)
                   for (int y = 0; y < sourceBitmap.Height; y++)
                   {
                       Color col = sourceBitmap.GetPixel(x, y);
                       g = col.G;
                       Color col1 = Color.FromArgb(col.A, g, g, g);
                       sourceBitmap.SetPixel(x, y, col1);
                   }
               bmpHSV1.UnlockBits(data);
               return sourceBitmap;
               #endregion
           }


        //---------------------------------------------
           enum ClrChannel : int { Blue, Green, Red, Gray };

       /*
        public static Bitmap GetChannel(Bitmap sourceBitmap, int iColor)	// [in] color plane to get          
           {
               Bitmap Nbmp = new Bitmap(sourceBitmap.Width, sourceBitmap.Height, PixelFormat.Format8bppIndexed);
               ColorPalette pal = Nbmp.Palette;
               for (int i = 0; i < 256; i++)
                   pal.Entries[i] = Color.FromArgb(i, i, i); ;
                   Nbmp.Palette = pal; // The crucial statement 

          //    if (PixelFormat.Format24bppRgb)
              {
                 //      ModelRGB_NewRed(sourceBitmap, Nbmp, iColor);
              }
              else        
                 //    ModelRGB_NewRed2(sourceBitmap, Nbmp, iColor);
               }
               return Nbmp;
           }
        */


           public static Bitmap ModelRGB_X(this Bitmap sourceBitmap, Bitmap Nbmp, Color rgb)
           {
               BitmapData sourceData = sourceBitmap.LockBits(new Rectangle(0, 0, sourceBitmap.Width, sourceBitmap.Height), ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
               BitmapData Ndata = Nbmp.LockBits(new Rectangle(0, 0, Nbmp.Width, Nbmp.Height), ImageLockMode.ReadWrite, PixelFormat.Format8bppIndexed);
               
               unsafe
               {
                   byte* Optr = (byte*)(sourceData.Scan0);
                   byte* Nptr = (byte*)(Ndata.Scan0);

                   for (int y = 0; y < sourceBitmap.Height; y++)
                   {
                       for (int x = 0; x < sourceBitmap.Width; x++)
                       {
                           byte b = *Optr; Optr++;
                           byte g = *Optr; Optr++;
                           byte r = *Optr; Optr++;

                           if (rgb == Color.Blue)
                           {
                               *Nptr = b; Nptr++;                                               
                           }
                           else if (rgb == Color.Red)
                           {
                               *Nptr = r; Nptr++;                           
                           }
                           else if (rgb == Color.Green)
                           {
                               *Nptr = g; Nptr++;
                           }
                           else if (rgb == Color.Gray)
                           {
                                 *Nptr = (byte)(((double)b + (double)g + (double)r) / 3.0); Nptr++;
                           }                    
                       }
                       Optr += sourceData.Stride - sourceData.Width * 3;
                       Nptr += Ndata.Stride - Ndata.Width;
                   }
                   sourceBitmap.UnlockBits(sourceData);
                   Nbmp.UnlockBits(Ndata);
               }
               return Nbmp;
           }

           public static Bitmap ModelRGB_NewRed( this Bitmap sourceBitmap, Bitmap Nbmp, int iColor)
           {
               BitmapData sourceData = sourceBitmap.LockBits(new Rectangle(0, 0, sourceBitmap.Width, sourceBitmap.Height), ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
               BitmapData Ndata = Nbmp.LockBits(new Rectangle(0, 0, Nbmp.Width, Nbmp.Height), ImageLockMode.ReadWrite, PixelFormat.Format8bppIndexed);
               
               /*ColorPalette pal = Nbmp.Palette;
               for (int i = 0; i < 256; i++)
                   pal.Entries[i] = Color.FromArgb(i, i, i);;
                   Nbmp.Palette = pal; */

               unsafe
               {
                   byte* Optr = (byte*)(sourceData.Scan0);
                   byte* Nptr = (byte*)(Ndata.Scan0);

                   for (int y = 0; y < sourceBitmap.Height; y++)
                   {
                       for (int x = 0; x < sourceBitmap.Width; x++)
                       {
                           byte b = *Optr; Optr++;
                           byte g = *Optr; Optr++;
                           byte r = *Optr; Optr++;

                           switch (iColor)
                           {
                               case (int)ClrChannel.Blue:
                                   *Nptr = b; Nptr++;
                                   break;
                               case (int)ClrChannel.Green:
                                   *Nptr = g; Nptr++;
                                   break;
                               case (int)ClrChannel.Red:
                                   *Nptr = r; Nptr++;
                                   break;
                               case (int)ClrChannel.Gray:
                                   *Nptr = (byte)(((double)b + (double)g + (double)r) / 3.0); Nptr++;
                                   break;
                           };
                       }
                       Optr += sourceData.Stride - sourceData.Width * 3;
                       Nptr += Ndata.Stride - Ndata.Width;
                   }
                   sourceBitmap.UnlockBits(sourceData);
                   Nbmp.UnlockBits(Ndata);
               }
                return Nbmp;
           }

        //-----------------
        /*
           private void ModelRGB_NewRed2(this Bitmap Obmp, Bitmap Nbmp, int iColor)
           {
               BitmapData Ndata = Nbmp.LockBits(new Rectangle(0, 0, Nbmp.Width, Nbmp.Height),ImageLockMode.ReadWrite, PixelFormat.Format8bppIndexed);

               unsafe
               {
                   byte* Nptr = (byte*)(Ndata.Scan0);

                   for (int y = 0; y < Obmp.Height; y++)
                   {
                       for (int x = 0; x < Obmp.Width; x++)
                       {
                           Color clr = Obmp.GetPixel(x, y);

                           switch (iColor)
                           {
                               case (int)ClrChannel.Blue:
                                   *Nptr = clr.B; Nptr++;
                                   break;

                               case (int)ClrChannel.Green:
                                   *Nptr = clr.G; Nptr++;
                                   break;

                               case (int)ClrChannel.Red:
                                   *Nptr = clr.R; Nptr++;
                                   break;

                               case (int)ClrChannel.Gray:
                                   *Nptr = (byte)(((double)clr.B + (double)clr.G + (double)clr.R) / 3.0); Nptr++;
                                   break;
                           };
                       }
                       Nptr += Ndata.Stride - Ndata.Width;
                   }
                   Nbmp.UnlockBits(Ndata);
               }
           }
        */
               /*
               //----------
                  double green = 0;

               for (int k = 0; k + 4 < pixelBuffer.Length; k += 4)
               {
                   green = pixelBuffer[0] = pixelBuffer[1] = pixelBuffer[2];
                   green += 3;

                   if (green > 255)
                   { green = 255; }
                   else if (green < 0)
                   { green = 0; }

                   pixelBuffer[k] = (byte)green;
               }
               Bitmap resultBitmap = new Bitmap(sourceBitmap.Width, sourceBitmap.Height);
               BitmapData resultData = resultBitmap.LockBits(new Rectangle(0, 0, resultBitmap.Width, resultBitmap.Height), ImageLockMode.WriteOnly, PixelFormat.Format32bppArgb);
               Marshal.Copy(pixelBuffer, 0, resultData.Scan0, pixelBuffer.Length);
            //   resultBitmap.UnlockBits(resultData);
               return resultBitmap;
           }
                */


        //-------------------------------------------



        ////-------------------
       /*
        public static Bitmap CopyToSquareCanvas(this Bitmap sourceBitmap, int canvasWidthLenght)
        {
            float ratio = 1.0f;
            int maxSide = sourceBitmap.Width > sourceBitmap.Height ? sourceBitmap.Width : sourceBitmap.Height;
            ratio = (float)maxSide / (float)canvasWidthLenght;
            Bitmap bitmapResult = (sourceBitmap.Width > sourceBitmap.Height ? new Bitmap(canvasWidthLenght, (int)(sourceBitmap.Height / ratio)) : new Bitmap((int)(sourceBitmap.Width / ratio), canvasWidthLenght));
            using (Graphics graphicsResult = Graphics.FromImage(bitmapResult))
            {
                graphicsResult.CompositingQuality = CompositingQuality.HighQuality;
                graphicsResult.InterpolationMode = InterpolationMode.HighQualityBicubic;
                graphicsResult.PixelOffsetMode = PixelOffsetMode.HighQuality;
                graphicsResult.DrawImage(sourceBitmap, new Rectangle(0, 0, bitmapResult.Width, bitmapResult.Height), new Rectangle(0, 0, sourceBitmap.Width, sourceBitmap.Height), GraphicsUnit.Pixel);
                graphicsResult.Flush();
            }
            return bitmapResult;
        }
        */

        public static Bitmap ConvolutionFilter<T>(this Bitmap sourceBitmap, T filter) 
            where T : ConvolutionFilterBase
        {
            BitmapData sourceData = sourceBitmap.LockBits(new Rectangle(0, 0, sourceBitmap.Width, sourceBitmap.Height), ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
            byte[] pixelBuffer = new byte[sourceData.Stride * sourceData.Height];
            byte[] resultBuffer = new byte[sourceData.Stride * sourceData.Height];
            Marshal.Copy(sourceData.Scan0, pixelBuffer, 0, pixelBuffer.Length);
            sourceBitmap.UnlockBits(sourceData);
            double blue = 0.0;
            double green = 0.0;
            double red = 0.0;
            int filterWidth = filter.FilterMatrix.GetLength(1);
            int filterHeight = filter.FilterMatrix.GetLength(0);
            int filterOffset = (filterWidth-1) / 2;
            int calcOffset = 0;
            int byteOffset = 0;

            for (int offsetY = filterOffset; offsetY < sourceBitmap.Height - filterOffset; offsetY++)
            {
                for (int offsetX = filterOffset; offsetX < sourceBitmap.Width - filterOffset; offsetX++)
                {
                    blue = 0;
                    green = 0;
                    red = 0;
                    byteOffset = offsetY * sourceData.Stride + offsetX * 4;

                    for (int filterY = -filterOffset; 
                        filterY <= filterOffset; filterY++)
                    {
                        for (int filterX = -filterOffset;
                            filterX <= filterOffset; filterX++)
                        {
                            calcOffset = byteOffset + (filterX * 4) + (filterY * sourceData.Stride);
                            blue += (double)(pixelBuffer[calcOffset]) * filter.FilterMatrix[filterY + filterOffset, filterX + filterOffset];
                            green += (double)(pixelBuffer[calcOffset + 1]) * filter.FilterMatrix[filterY + filterOffset, filterX + filterOffset];
                            red += (double)(pixelBuffer[calcOffset + 2]) * filter.FilterMatrix[filterY + filterOffset, filterX + filterOffset];
                        }
                    }
                    blue = filter.Factor * blue + filter.Bias;
                    green = filter.Factor * green + filter.Bias;
                    red = filter.Factor * red + filter.Bias;

                    if (blue > 255)
                    { blue = 255; }
                    else if (blue < 0)
                    { blue = 0; }

                    if (green > 255)
                    { green = 255; }
                    else if (green < 0)
                    { green = 0; }

                    if (red > 255)
                    { red = 255; }
                    else if (red < 0)
                    { red = 0; }

                    resultBuffer[byteOffset] = (byte)(blue);
                    resultBuffer[byteOffset + 1] = (byte)(green);
                    resultBuffer[byteOffset + 2] = (byte)(red);
                    resultBuffer[byteOffset + 3] = 255;
                }
            }
            Bitmap resultBitmap = new Bitmap(sourceBitmap.Width, sourceBitmap.Height);
            BitmapData resultData = resultBitmap.LockBits(new Rectangle(0, 0,resultBitmap.Width, resultBitmap.Height), ImageLockMode.WriteOnly, PixelFormat.Format32bppArgb);
            Marshal.Copy(resultBuffer, 0, resultData.Scan0, resultBuffer.Length);
            resultBitmap.UnlockBits(resultData);
            return resultBitmap;
        }
        
        //-----------------------
        public static Bitmap ConvolutionFilter2<T>(this Bitmap sourceBitmap, T filter, int metod)
            where T : ConvolutionFilterBase
        {
            BitmapData sourceData = sourceBitmap.LockBits(new Rectangle(0, 0, sourceBitmap.Width, sourceBitmap.Height), ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
            byte[] pixelBuffer = new byte[sourceData.Stride * sourceData.Height];
            byte[] resultBuffer = new byte[sourceData.Stride * sourceData.Height];
            Marshal.Copy(sourceData.Scan0, pixelBuffer, 0, pixelBuffer.Length);
            sourceBitmap.UnlockBits(sourceData);
            double blue = 0.0;
            double green = 0.0;
            double red = 0.0;
            /*
            double blueMin = 0.0;
            double greenMin = 0.0;
            double redMin = 0.0;

            double blueMax = 255.0;
            double greenMax = 255.0;
            double redMax = 255.0;
            */
            int filterWidth = filter.FilterMatrix.GetLength(1);
            int filterHeight = filter.FilterMatrix.GetLength(0);
            int filterOffset = (filterWidth - 1) / 2;
            int calcOffset = 0;
            int byteOffset = 0;

            for (int offsetY = filterOffset; offsetY < sourceBitmap.Height - filterOffset; offsetY++)
            {
                for (int offsetX = filterOffset; offsetX < sourceBitmap.Width - filterOffset; offsetX++)
                {
                    blue = 0;
                    green = 0;
                    red = 0;
                    byteOffset = offsetY * sourceData.Stride + offsetX * 4;

                    for (int filterY = -filterOffset;
                        filterY <= filterOffset; filterY++)
                    {
                        for (int filterX = -filterOffset;
                            filterX <= filterOffset; filterX++)
                        {
                            calcOffset = byteOffset + (filterX * 4) + (filterY * sourceData.Stride);
                            blue += (double)(pixelBuffer[calcOffset]) * filter.FilterMatrix[filterY + filterOffset, filterX + filterOffset];
                            green += (double)(pixelBuffer[calcOffset + 1]) * filter.FilterMatrix[filterY + filterOffset, filterX + filterOffset];
                            red += (double)(pixelBuffer[calcOffset + 2]) * filter.FilterMatrix[filterY + filterOffset, filterX + filterOffset];
                        }
                    }
                    blue = filter.Factor * blue + filter.Bias;
                    green = filter.Factor * green + filter.Bias;
                    red = filter.Factor * red + filter.Bias;

                    if (metod == 1)
                    {
                        if (blue > 255)
                        { blue = 255; }
                        else if (blue < 0)
                        { blue = 0; }

                        if (green > 255)
                        { green = 255; }
                        else if (green < 0)
                        { green = 0; }

                        if (red > 255)
                        { red = 255; }
                        else if (red < 0)
                        { red = 0; }                                      
                    }                                  
                    
                    if (metod == 2)
                    {
                        if (blue > 255)
                        { blue = 255; }
                        else if (blue < 0)
                        { blue = 0; }
                        else
                        { blue = 128; }

                        if (green > 255)
                        { green = 255; }
                        else if (green < 0)
                        { green = 0; }
                        else
                        { green = 128; }

                        if (red > 255)
                        { red = 255; }
                        else if (red < 0)
                        { red = 0; }
                        else
                        { red = 128; }
                    }

                    /*
                    max2d(sourceBitmap, blueMin, redMin, greenMax);
              //      return (byte)(((double)(value - min)) / ((double)(max - min)));
                    if (metod == 3)
                    {
                       ((blue - blueMin) / (blueMax - blueMin));
                       ((red - redMin) / (redMax - redMin));
                       ((green - greenMin) / (greenMax - greenMin));                      
             }
                     */


                    resultBuffer[byteOffset] = (byte)(blue);
                    resultBuffer[byteOffset + 1] = (byte)(green);
                    resultBuffer[byteOffset + 2] = (byte)(red);
                    resultBuffer[byteOffset + 3] = 255;
                }
            }
            Bitmap resultBitmap = new Bitmap(sourceBitmap.Width, sourceBitmap.Height);
            BitmapData resultData = resultBitmap.LockBits(new Rectangle(0, 0, resultBitmap.Width, resultBitmap.Height), ImageLockMode.WriteOnly, PixelFormat.Format32bppArgb);
            Marshal.Copy(resultBuffer, 0, resultData.Scan0, resultBuffer.Length);
            resultBitmap.UnlockBits(resultData);
            return resultBitmap;
        }

        public static Bitmap BitwiseBlend(this Bitmap sourceBitmap, Bitmap blendBitmap, BitwiseBlendType blendTypeBlue, BitwiseBlendType blendTypeGreen, BitwiseBlendType blendTypeRed)
        {
            #region BitwiseBlend - Operacje Wieloargumentowe Logiczne
            BitmapData sourceData = sourceBitmap.LockBits(new Rectangle(0, 0, sourceBitmap.Width, sourceBitmap.Height), ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
            byte[] pixelBuffer = new byte[sourceData.Stride * sourceData.Height];
            Marshal.Copy(sourceData.Scan0, pixelBuffer, 0, pixelBuffer.Length);
            sourceBitmap.UnlockBits(sourceData);

            BitmapData blendData = blendBitmap.LockBits(new Rectangle(0, 0, blendBitmap.Width, blendBitmap.Height), ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
            byte[] blendBuffer = new byte[blendData.Stride * blendData.Height];
            Marshal.Copy(blendData.Scan0, blendBuffer, 0, blendBuffer.Length);
            blendBitmap.UnlockBits(blendData);

            int blue = 0, green = 0, red = 0;

            for (int k = 0; (k + 4 < pixelBuffer.Length) && (k + 4 < blendBuffer.Length); k += 4)
            {
                if (blendTypeBlue == BitwiseBlendType.AND)
                {
                    blue = pixelBuffer[k] & blendBuffer[k];
                }
                else if (blendTypeBlue == BitwiseBlendType.OR)
                {
                    blue = pixelBuffer[k] | blendBuffer[k];
                }
                else if (blendTypeBlue == BitwiseBlendType.XOR)
                {
                    blue = pixelBuffer[k] ^ blendBuffer[k];
                }
                else if (blendTypeBlue == BitwiseBlendType.XNOR)
                {
                    blue = 255-( pixelBuffer[k] ^ blendBuffer[k]);
                }

                if (blendTypeGreen == BitwiseBlendType.AND)
                {
                    green = pixelBuffer[k + 1] & blendBuffer[k + 1];
                }
                else if (blendTypeGreen == BitwiseBlendType.OR)
                {
                    green = pixelBuffer[k + 1] | blendBuffer[k + 1];
                }
                else if (blendTypeGreen == BitwiseBlendType.XOR)
                {
                    green = pixelBuffer[k + 1] ^ blendBuffer[k + 1];
                }
                else if (blendTypeGreen == BitwiseBlendType.XNOR)
                {
                    green = 255-(pixelBuffer[k + 1] ^ blendBuffer[k + 1]);
                }

                if (blendTypeRed == BitwiseBlendType.AND)
                {
                    red = pixelBuffer[k + 2] & blendBuffer[k + 2];
                }
                else if (blendTypeRed == BitwiseBlendType.OR)
                {
                    red = pixelBuffer[k + 2] | blendBuffer[k + 2];
                }
                else if (blendTypeRed == BitwiseBlendType.XOR)
                {
                    red = pixelBuffer[k + 2] ^ blendBuffer[k + 2];
                }
                else if (blendTypeRed == BitwiseBlendType.XNOR)
                {
                    red = 255 -( pixelBuffer[k + 2] ^ blendBuffer[k + 2]);
                }

                if (blue < 0)
                { blue = 0; }
                else if (blue > 255)
                { blue = 255; }

                if (green < 0)
                { green = 0; }
                else if (green > 255)
                { green = 255; }

                if (red < 0)
                { red = 0; }
                else if (red > 255)
                { red = 255; }

                pixelBuffer[k] = (byte)blue;
                pixelBuffer[k + 1] = (byte)green;
                pixelBuffer[k + 2] = (byte)red;
            }

            Bitmap resultBitmap = new Bitmap(sourceBitmap.Width, sourceBitmap.Height);
            BitmapData resultData = resultBitmap.LockBits(new Rectangle(0, 0, resultBitmap.Width, resultBitmap.Height), ImageLockMode.WriteOnly, PixelFormat.Format32bppArgb);
            Marshal.Copy(pixelBuffer, 0, resultData.Scan0, pixelBuffer.Length);
            resultBitmap.UnlockBits(resultData);
            return resultBitmap;
            #endregion
        }

        public enum BitwiseBlendType
        {
            #region BitwiseBlendType
            None,
            OR,
            AND,
            XOR,
            XNOR
            #endregion
        }


        public static void max2d(this Bitmap sourceBitmap, out double blueMax, out double redMax, out double greenMax)
        {
         //   double blueMax =  Double.MinValue;
          blueMax = 0.0;
          redMax = 0.0;
          greenMax = 0.0;

            double blue = 0.0;
            double red = 0.0;
            double green = 0.0;

            for (int x = 0; x < sourceBitmap.Width; x++)
                for (int y = 0; y < sourceBitmap.Height; y++)               
                    if (blue > blueMax)                  
                    {  blueMax = blue; }
                    if (red > redMax)
                     { redMax = red; }
                  if (green > greenMax)
                    { greenMax = green; }
        }

        //----------------------------------------------------------------------------------------------------------------
        public static Bitmap ProgowanieHSV(this Bitmap sourceBitmap, float threshold)
        {
            #region ProgowanieHSV
            Bitmap bmpHSV = new Bitmap(sourceBitmap);
            int new_height = sourceBitmap.Height;
            int new_width = sourceBitmap.Width;
            while (new_height > 300 || new_width > 300)
            {
                new_height = (int)(new_height / 1.05f);
                new_width = (int)(new_width / 1.05f);
            }
            BitmapData data = bmpHSV.LockBits(new System.Drawing.Rectangle(0, 0, new_width, new_height), ImageLockMode.ReadWrite, PixelFormat.Format32bppArgb);

            unsafe
            {
                byte* pBase = (byte*)data.Scan0.ToPointer();

                for (int x = 0; x < sourceBitmap.Width; x++)
                {
                    for (int y = 0; y < sourceBitmap.Height; y++)
                    {
                        double h, s, v;
                        hsv.ColorToHSV(sourceBitmap.GetPixel(x, y), out h, out s, out v);
                        if (v > threshold)
                        {
                            sourceBitmap.SetPixel(x, y, hsv.ColorFromHSV(h, s, 0));
                        }
                        else
                        {
                            sourceBitmap.SetPixel(x, y, hsv.ColorFromHSV(h, s, 1));
                        }
                    }
                }
                bmpHSV.UnlockBits(data);
                return sourceBitmap;
            }

            #endregion
        }


        public static Bitmap progowanie_hsv( this Bitmap src, double prog)
        {

            BitmapProcessing.FastBitmap fobrazek_bitmapa = new BitmapProcessing.FastBitmap(src);
            fobrazek_bitmapa.LockImage();


            for (int x = 0; x < src.Width; x++)
            {
                for (int y = 0; y < src.Height; y++)
                {
                    double h, s, v;
                    hsv.ColorToHSV(fobrazek_bitmapa.GetPixel(x, y), out h, out s, out v);

                    if (v > prog)
                    {
                        fobrazek_bitmapa.SetPixel(x, y, hsv.ColorFromHSV(h, s, 0));
                    }
                    else
                    {
                        fobrazek_bitmapa.SetPixel(x, y, hsv.ColorFromHSV(h, s, 1));
                    }
                }
            }


            fobrazek_bitmapa.UnlockImage();
            return src;
        }

        //----------------------------
        public static Bitmap ProgowanieHSV_new(this Bitmap sourceBitmap, float threshold)
        {
            #region ProgowanieHSV
            Bitmap bmpHSV = new Bitmap(sourceBitmap);
            int new_height = sourceBitmap.Height;
            int new_width = sourceBitmap.Width;
            while (new_height > 300 || new_width > 300)
            {
                new_height = (int)(new_height / 1.05f);
                new_width = (int)(new_width / 1.05f);
            }
            BitmapData data = bmpHSV.LockBits(new System.Drawing.Rectangle(0, 0, new_width, new_height), ImageLockMode.ReadWrite, PixelFormat.Format32bppArgb);
          //  BitmapData sourceData = sourceBitmap.LockBits(new Rectangle(0, 0, sourceBitmap.Width, sourceBitmap.Height), ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
          //  byte[] pixelBuffer = new byte[sourceData.Stride * sourceData.Height];
           // Marshal.Copy(sourceData.Scan0, pixelBuffer, 0, pixelBuffer.Length);
         //   bmpHSV.UnlockBits(data);

            unsafe
            {
                byte* pBase = (byte*)data.Scan0.ToPointer();

                for (int x = 0; x < sourceBitmap.Width; x++)
                {
                    for (int y = 0; y < sourceBitmap.Height; y++)
                    {
                        double h, s, v;
                        hsv.ColorToHSV(sourceBitmap.GetPixel(x, y), out h, out s, out v);
                        if (v > threshold)
                        {
                            sourceBitmap.SetPixel(x, y, hsv.ColorFromHSV(h, s, 0));
                        }
                        else
                        {
                            sourceBitmap.SetPixel(x, y, hsv.ColorFromHSV(h, s, 1));
                        }
                    }
                }

              bmpHSV.UnlockBits(data);
              return sourceBitmap;
            }

            #endregion
        }
   
 

        //--------------------------------
        public static Bitmap JasnoscRGB2(this Bitmap sourceBitmap, int threshold)
        {
            BitmapData sourceData = sourceBitmap.LockBits(new Rectangle(0, 0, sourceBitmap.Width, sourceBitmap.Height), ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
            byte[] pixelBuffer = new byte[sourceData.Stride * sourceData.Height];
            Marshal.Copy(sourceData.Scan0, pixelBuffer, 0, pixelBuffer.Length);
            sourceBitmap.UnlockBits(sourceData);

         //   double contrastLevel = Math.Pow((100.0 + threshold) / 100.0, 2);
            double blue = 0;
            double green = 0;
            double red = 0;

            for (int k = 0; k + 4 < pixelBuffer.Length; k += 4)
            {
                blue = pixelBuffer[k] + threshold;
                green = pixelBuffer[k + 1] + threshold;
                red = pixelBuffer[k + 2] + threshold;

                if (blue < 0)
                { blue = 0; }
                else if (blue > 255)
                { blue = 255; }
  
                if (green > 255)
                { green = 255; }
                else if (green < 0)
                { green = 0; }
        
                if (red > 255)
                { red = 255; }
                else if (red < 0)
                { red = 0; }
 
                pixelBuffer[k] = (byte)blue;
                pixelBuffer[k + 1] = (byte)green;
                pixelBuffer[k + 2] = (byte)red;
            }
            Bitmap resultBitmap = new Bitmap(sourceBitmap.Width, sourceBitmap.Height);
            BitmapData resultData = resultBitmap.LockBits(new Rectangle(0, 0, resultBitmap.Width, resultBitmap.Height), ImageLockMode.WriteOnly, PixelFormat.Format32bppArgb);
            Marshal.Copy(pixelBuffer, 0, resultData.Scan0, pixelBuffer.Length);
            resultBitmap.UnlockBits(resultData);
            return resultBitmap;
        }
    }

        ///////---------------------


        
        ////
        /*
        public static Bitmap AlgorytmPiramida(this Bitmap sourceBitmap, int n, int m)
        {
            BitmapData sourceData = sourceBitmap.LockBits(new Rectangle(0, 0, sourceBitmap.Width, sourceBitmap.Height), ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
            byte[] pixelBuffer = new byte[sourceData.Stride * sourceData.Height];
            Marshal.Copy(sourceData.Scan0, pixelBuffer, 0, pixelBuffer.Length);
            sourceBitmap.UnlockBits(sourceData);

            int[][] levels = new int[256][];
       
            for (int x = 0; x < N; x++)
                for (int y = 0; y < N; y++)
                  //   sourceBitmap.GetPixel[x][y] = 0;
                    sourceBitmap.GetPixel(x, y)= 0;
            N /= 2;

            for (int x = 0; x < N; x++)
                for (int y = 0; y < N; y++)
                    sourceBitmap.GetPixel(x, y) = (sourceBitmap(2*x, 2*y) || sourceBitmap(2*x+1, 2*y) || 
                    sourceBitmap(2*x, 2*y+1) || sourceBitmap(2*x+1, 2*y+1);
            
            int NN, yy;
            int NNO = 0;

            do 
            {
            NNO = NN;
            NN +=NN;
            N/=2;
             for (int x = 0; x < N; x++)
                for (int y = 0; y < N; y++)
                   sourceBitmap.GetPixel(x, y+NNO) = (sourceBitmap(2*x, 2*y+NNO) || sourceBitmap(2*x+1, 2*y+NNO) || 
                   sourceBitmap(2*x, 2*y+1+NNO) || sourceBitmap(2*x+1, 2*y+1+NNO);        
            }
            while(N!=1);
                     
        }
    
        */

        /*

                for (int k = 0; k + 4 < pixelBuffer.Length; k += 4)
                {
                    blue = pixelBuffer[k] + (pixelBuffer.Length / 256.0);
                    green = pixelBuffer[k + 1] + (pixelBuffer.Length / 256.0);
                    red = pixelBuffer[k + 2] + (pixelBuffer.Length / 256.0);

                    if (blue < 0)
                    { blue = 0; }
                    else if (blue > pixelBuffer.Length - 1)
                    {
                        blue = pixelBuffer.Length - 1;
                    }

                    if (green < 0)
                    { green = 0; }
                    else if (green > pixelBuffer.Length - 1)
                    {
                        green = pixelBuffer.Length - 1;
                    }

                    if (red < 0)
                    { red = 0; }
                    else if (red > pixelBuffer.Length - 1)
                    {
                        red = pixelBuffer.Length - 1;
                    }

                    pixelBuffer[k] = (byte)blue;
                    pixelBuffer[k + 1] = (byte)green;
                    pixelBuffer[k + 2] = (byte)red;
                }
                Bitmap resultBitmap = new Bitmap(sourceBitmap.Width, sourceBitmap.Height);
                BitmapData resultData = resultBitmap.LockBits(new Rectangle(0, 0, resultBitmap.Width, resultBitmap.Height), ImageLockMode.WriteOnly, PixelFormat.Format32bppArgb);
                Marshal.Copy(pixelBuffer, 0, resultData.Scan0, pixelBuffer.Length);
                resultBitmap.UnlockBits(resultData);
                return resultBitmap;
            }
        */



}




    




       /*
        public Bitmap Bineryzacja(Bitmap  bmp, int wartosc)
        {
            Bitmap bmp1 = GetArgbCopy(bmp);
            BitmapData bmpData = bmp.LockBits(new Rectangle(0, 0, bmp.Width, bmp.Height), ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);

            
            for (int x = 0; x < bmp.Width; x++)
                for (int y = 0; y < bmp.Height; y++)
                {
                    Color px = bmp.GetPixel(x, y);
                    if ((int)((px.R * 0.3) + (px.G * 0.59) + (px.B * 0.11)) <= wartosc)
                    {
                        Color ob = Color.FromArgb(px.A, 0, 0, 0);
                        bmp.SetPixel(x, y, ob);
                    }
                    else
                    {
                        Color ob = Color.FromArgb(px.A, 255, 255, 255);
                        bmp.SetPixel(x, y, ob);
                    }
                }
            return bmp;
        }
       */





