﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace APO_BB_DK_v012013
{
    class KonwersjaRGB_HSV
    {
        public static void ColorToHSV(Color color, out double hue, out double saturation, out double value)
        {
            int max = Math.Max(color.R, Math.Max(color.G, color.B));
            int min = Math.Min(color.R, Math.Min(color.G, color.B));

            hue = color.GetHue();
            saturation = (max == 0) ? 0 : 1d - (1d * min / max);
            value = max / 255d;
        }

        public static Color ColorFromHSV(double hue, double saturation, double value)
        {
            int hi = Convert.ToInt32(Math.Floor(hue / 60)) % 6;
            double f = hue / 60 - Math.Floor(hue / 60);

            value = value * 255;
            int v = Convert.ToInt32(value);
            int p = Convert.ToInt32(value * (1 - saturation));
            int q = Convert.ToInt32(value * (1 - f * saturation));
            int t = Convert.ToInt32(value * (1 - (1 - f) * saturation));

            if (hi == 0)
                return Color.FromArgb(255, v, t, p);
            else if (hi == 1)
                return Color.FromArgb(255, q, v, p);
            else if (hi == 2)
                return Color.FromArgb(255, p, v, t);
            else if (hi == 3)
                return Color.FromArgb(255, p, q, v);
            else if (hi == 4)
                return Color.FromArgb(255, t, p, v);
            else
                return Color.FromArgb(255, v, p, q);
        }


        public static Color HSVtoRGB(float hue, float saturation, float value, float alpha)
        {
            while (hue > 1f) { hue -= 1f; }
            while (hue < 0f) { hue += 1f; }
            while (saturation > 1f) { saturation -= 1f; }
            while (saturation < 0f) { saturation += 1f; }
            while (value > 1f) { value -= 1f; }
            while (value < 0f) { value += 1f; }
            if (hue > 0.999f) { hue = 0.999f; }
            if (hue < 0.001f) { hue = 0.001f; }
            if (saturation > 0.999f) { saturation = 0.999f; }
            if (saturation < 0.001f) { return new Color(value * 255f, value * 255f, value * 255f); }
            if (value > 0.999f) { value = 0.999f; }
            if (value < 0.001f) { value = 0.001f; }

            float h6 = hue * 6f;
            if (h6 == 6f) { h6 = 0f; }
            int ihue = (int)(h6);
            float p = value * (1f - saturation);
            float q = value * (1f - (saturation * (h6 - (float)ihue)));
            float t = value * (1f - (saturation * (1f - (h6 - (float)ihue))));
            switch (ihue)
            {
                case 0:
                    return new Color(value, t, p, alpha);
                case 1:
                    return new Color(q, value, p, alpha);
                case 2:
                    return new Color(p, value, t, alpha);
                case 3:
                    return new Color(p, q, value, alpha);
                case 4:
                    return new Color(t, p, value, alpha);
                default:
                    return new Color(value, p, q, alpha);
            }
        }

       
    }
}
        
        
        
       /* 
        double var_R, var_G, var_B, var_Max, var_Min, del_Max;                
        var_R = ( R / 255);                     // RGB od 0 do 255
        var_G = ( G / 255);
        var_B = ( B / 255);
        var_Min = min (var_R, var_G, var_B);     // min. wartość RGB 
        var_Max = max (var_R, var_G, var_B);     // Max. wartość RGB 
        del_Max = var_Max - var_Min;              // Delta RGB wartość 

        V = var_Max jeśli (del_Max == 0);        // To jest szary, nie chroma ... 
       { H  = 0;              
        
        // Wyniki /
        HSV od 0 do 1 S  = 0 } else            // dane chromatyczna ... 
        { 
            S = del_Max / var_Max;
            del_R = (((var_Max - var_R) / 6) + (del_Max / 2)) / del_Max;
            del_G = (((var_Max - var_G) / 6) + ( del_Max / 2)) / del_Max    
            del_B = (((var_Max - var_B) / 6) + (del_Max / 2)) / del_Max    
        jeśli (var_R == var_Max) H  = del_B - del_G    
        else if (var_G == var_Max)  H  = (1/3) + del_R - del_B    
         if (var_B == var_Max)  H  = (2/3) + del_G - del_R    
            if ( H  <0) H  + = 1    
        , jeżeli ( H  > 1) H  - = 1 }
        */


    
