﻿/*
 * The Following Code was developed by Dewald Esterhuizen
 * View Documentation at: http://softwarebydefault.com
 * Licensed under Ms-PL 
*/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace APO_BB_DK_v012013
{
  //  class MatrixNew    

        #region MatrixNew
        #endregion

    public class Blur3x3Filter : ConvolutionFilterBase
    {
        #region Blur3x3Filter : ConvolutionFilterBase

        public override string FilterName
        {
            get { return "Blur3x3Filter"; }
        }

        private double factor = 1.0;
        public override double Factor
        {
            get { return factor; }
        }

        private double bias = 0.0;
        public override double Bias
        {
            get { return bias; }
        }

        private double[,] filterMatrix =
            new double[,] { { 0.0, 0.2, 0.0, }, 
                            { 0.2, 0.2, 0.2, }, 
                            { 0.0, 0.2, 0.2, }, };

        public override double[,] FilterMatrix
        {
            get { return filterMatrix; }
        }
        #endregion
    }

    public class Blur5x5Filter : ConvolutionFilterBase
    {
        #region Blur5x5Filter : ConvolutionFilterBase

        public override string FilterName
        {
            get { return "Blur5x5Filter"; }
        }

        private double factor = 1.0 / 13.0;
        public override double Factor
        {
            get { return factor; }
        }

        private double bias = 0.0;
        public override double Bias
        {
            get { return bias; }
        }

        private double[,] filterMatrix =
            new double[,] { { 0, 0, 1, 0, 0, }, 
                            { 0, 1, 1, 1, 0, }, 
                            { 1, 1, 1, 1, 1, },
                            { 0, 1, 1, 1, 0, },
                            { 0, 0, 1, 0, 0, }, };

        public override double[,] FilterMatrix
        {
            get { return filterMatrix; }
        }

        #endregion
    }

    public class Gaussian3x3BlurFilter : ConvolutionFilterBase
    {
        #region  Gaussian3x3BlurFilter : ConvolutionFilterBase

        public override string FilterName
        {
            get { return "Gaussian3x3BlurFilter"; }
        }

        private double factor = 1.0 / 16.0;
        public override double Factor
        {
            get { return factor; }
        }

        private double bias = 0.0;
        public override double Bias
        {
            get { return bias; }
        }

        private double[,] filterMatrix =
            new double[,] { { 1, 2, 1, }, 
                            { 2, 4, 2, }, 
                            { 1, 2, 1, }, };

        public override double[,] FilterMatrix
        {
            get { return filterMatrix; }
        }
         #endregion
    }

    public class Gaussian5x5BlurFilter : ConvolutionFilterBase
    {
        #region Gaussian5x5BlurFilter : ConvolutionFilterBase

        public override string FilterName
        {
            get { return "Gaussian5x5BlurFilter"; }
        }

        private double factor = 1.0 / 159.0;
        public override double Factor
        {
            get { return factor; }
        }

        private double bias = 0.0;
        public override double Bias
        {
            get { return bias; }
        }

        private double[,] filterMatrix =
            new double[,] { { 2, 04, 05, 04, 2 }, 
                            { 4, 09, 12, 09, 4 }, 
                            { 5, 12, 15, 12, 5 },
                            { 4, 09, 12, 09, 4 },
                            { 2, 04, 05, 04, 2 }, };

        public override double[,] FilterMatrix
        {
            get { return filterMatrix; }
        }

        #endregion
    }

    public class MotionBlurFilter : ConvolutionFilterBase
    {
        #region MotionBlurFilter : ConvolutionFilterBase

        public override string FilterName
        {
            get { return "MotionBlurFilter"; }
        }

        private double factor = 1.0 / 18.0;
        public override double Factor
        {
            get { return factor; }
        }

        private double bias = 0.0;
        public override double Bias
        {
            get { return bias; }
        }

        private double[,] filterMatrix =
            new double[,] { {1, 0, 0, 0, 0, 0, 0, 0, 1,},
                            {0, 1, 0, 0, 0, 0, 0, 1, 0,},
                            {0, 0, 1, 0, 0, 0, 1, 0, 0,},
                            {0, 0, 0, 1, 0, 1, 0, 0, 0,},
                            {0, 0, 0, 0, 1, 0, 0, 0, 0,},
                            {0, 0, 0, 1, 0, 1, 0, 0, 0,},
                            {0, 0, 1, 0, 0, 0, 1, 0, 0,},
                            {0, 1, 0, 0, 0, 0, 0, 1, 0,},
                            {1, 0, 0, 0, 0, 0, 0, 0, 1,}, };

        public override double[,] FilterMatrix
        {
            get { return filterMatrix; }
        }

        #endregion
    }

    public class MotionBlurLeftToRightFilter : ConvolutionFilterBase
    {
        #region MotionBlurLeftToRightFilter : ConvolutionFilterBase 

        public override string FilterName
        {
            get { return "MotionBlurLeftToRightFilter"; }
        }

        private double factor = 1.0 / 9.0;
        public override double Factor
        {
            get { return factor; }
        }

        private double bias = 0.0;
        public override double Bias
        {
            get { return bias; }
        }

        private double[,] filterMatrix =
            new double[,] { {1, 0, 0, 0, 0, 0, 0, 0, 0,},
                            {0, 1, 0, 0, 0, 0, 0, 0, 0,},
                            {0, 0, 1, 0, 0, 0, 0, 0, 0,},
                            {0, 0, 0, 1, 0, 0, 0, 0, 0,},
                            {0, 0, 0, 0, 1, 0, 0, 0, 0,},
                            {0, 0, 0, 0, 0, 1, 0, 0, 0,},
                            {0, 0, 0, 0, 0, 0, 1, 0, 0,},
                            {0, 0, 0, 0, 0, 0, 0, 1, 0,},
                            {0, 0, 0, 0, 0, 0, 0, 0, 1,}, };

        public override double[,] FilterMatrix
        {
            get { return filterMatrix; }
        }

            #endregion
    }

    public class MotionBlurRightToLeftFilter : ConvolutionFilterBase
    {
        #region MotionBlurRightToLeftFilter : ConvolutionFilterBase

        public override string FilterName
        {
            get { return "MotionBlurRightToLeftFilter"; }
        }

        private double factor = 1.0 / 9.0;
        public override double Factor
        {
            get { return factor; }
        }

        private double bias = 0.0;
        public override double Bias
        {
            get { return bias; }
        }

        private double[,] filterMatrix =
            new double[,] { {0, 0, 0, 0, 0, 0, 0, 0, 1,},
                            {0, 0, 0, 0, 0, 0, 0, 1, 0,},
                            {0, 0, 0, 0, 0, 0, 1, 0, 0,},
                            {0, 0, 0, 0, 0, 1, 0, 0, 0,},
                            {0, 0, 0, 0, 1, 0, 0, 0, 0,},
                            {0, 0, 0, 1, 0, 0, 0, 0, 0,},
                            {0, 0, 1, 0, 0, 0, 0, 0, 0,},
                            {0, 1, 0, 0, 0, 0, 0, 0, 0,},
                            {1, 0, 0, 0, 0, 0, 0, 0, 0,}, };

        public override double[,] FilterMatrix
        {
            get { return filterMatrix; }
        }
        #endregion
    }

    public class HighPass3x3Filter : ConvolutionFilterBase
    {
        #region HighPass3x3Filter : ConvolutionFilterBase

        public override string FilterName
        {
            get { return "HighPass3x3Filter"; }
        }

        private double factor = 1.0 / 16.0;
        public override double Factor
        {
            get { return factor; }
        }

        private double bias = 128.0;
        public override double Bias
        {
            get { return bias; }
        }

        private double[,] filterMatrix =
            new double[,] { { -1, -2, -1, }, 
                            { -2, 12, -2, }, 
                            { -1, -2, -1, }, };

        public override double[,] FilterMatrix
        {
            get { return filterMatrix; }
        }
            #endregion
    }

    public class EmbossFilter : ConvolutionFilterBase
    {
        #region  EmbossFilter : ConvolutionFilterBase

        public override string FilterName
        {
            get { return "EmbossFilter"; }
        }

        private double factor = 1.0;
        public override double Factor
        {
            get { return factor; }
        }

        private double bias = 128.0;
        public override double Bias
        {
            get { return bias; }
        }

        private double[,] filterMatrix =
            new double[,] { { 2, 0, 0, }, 
                            { 0,-1, 0, }, 
                            { 0, 0,-1, }, };

        public override double[,] FilterMatrix
        {
            get { return filterMatrix; }
        }
        #endregion
    }

    public class Emboss45DegreeFilter : ConvolutionFilterBase
    {
        #region Emboss45DegreeFilter : ConvolutionFilterBase

        public override string FilterName
        {
            get { return "Emboss45DegreeFilter"; }
        }

        private double factor = 1.0;
        public override double Factor
        {
            get { return factor; }
        }

        private double bias = 128.0;
        public override double Bias
        {
            get { return bias; }
        }

        private double[,] filterMatrix =
            new double[,] { { -1, -1, 0, }, 
                            { -1,  0, 1, }, 
                            {  0,  1, 1, }, };

        public override double[,] FilterMatrix
        {
            get { return filterMatrix; }
        }
        #endregion
    }

    public class EmbossTopLeftBottomRightFilter : ConvolutionFilterBase
    {
        #region EmbossTopLeftBottomRightFilter : ConvolutionFilterBase

        public override string FilterName
        {
            get { return "EmbossTopLeftBottomRightFilter"; }
        }

        private double factor = 1.0;
        public override double Factor
        {
            get { return factor; }
        }

        private double bias = 128.0;
        public override double Bias
        {
            get { return bias; }
        }

        private double[,] filterMatrix =
            new double[,] { { -1, 0, 0, }, 
                            {  0, 0, 0, }, 
                            {  0, 0, 1, }, };

        public override double[,] FilterMatrix
        {
            get { return filterMatrix; }
        }
        #endregion
    }

    public class IntenseEmbossFilter : ConvolutionFilterBase
    {
        #region IntenseEmbossFilter : ConvolutionFilterBase

        public override string FilterName
        {
            get { return "IntenseEmbossFilter"; }
        }

        private double factor = 1.0;
        public override double Factor
        {
            get { return factor; }
        }

        private double bias = 128.0;
        public override double Bias
        {
            get { return bias; }
        }

        private double[,] filterMatrix =
            new double[,] { { -1, -1, -1, -1,  0, }, 
                            { -1, -1, -1,  0,  1, }, 
                            { -1, -1,  0,  1,  1, },
                            { -1,  0,  1,  1,  1, },
                            {  0,  1,  1,  1,  1, }, };

        public override double[,] FilterMatrix
        {
            get { return filterMatrix; }
        }
        #endregion
    }

    public class SharpenFilter : ConvolutionFilterBase
    {
        #region SharpenFilter : ConvolutionFilterBase

        public override string FilterName
        {
            get { return "SharpenFilter"; }
        }

        private double factor = 1.0;
        public override double Factor
        {
            get { return factor; }
        }

        private double bias = 0.0;
        public override double Bias
        {
            get { return bias; }
        }

        private double[,] filterMatrix =
            new double[,] { { -1, -1, -1, }, 
                            { -1,  9, -1, }, 
                            { -1, -1, -1, }, };

        public override double[,] FilterMatrix
        {
            get { return filterMatrix; }
        }
        #endregion
    }

    public class Sharpen3x3Filter : ConvolutionFilterBase
    {
        #region Sharpen3x3Filter : ConvolutionFilterBase

        public override string FilterName
        {
            get { return "Sharpen3x3Filter"; }
        }

        private double factor = 1.0;
        public override double Factor
        {
            get { return factor; }
        }

        private double bias = 0.0;
        public override double Bias
        {
            get { return bias; }
        }

        private double[,] filterMatrix =
            new double[,] { { 0,-1, 0, }, 
                            { -1, 5, -1, }, 
                            { 0,-1, 0, }, };

        public override double[,] FilterMatrix
        {
            get { return filterMatrix; }
        }
        #endregion
    }

    public class Sharpen3x3FactorFilter : ConvolutionFilterBase
    {
        #region Sharpen3x3FactorFilter : ConvolutionFilterBase

        public override string FilterName
        {
            get { return "Sharpen3x3FactorFilter"; }
        }

        private double factor = 1.0 / 3.0;
        public override double Factor
        {
            get { return factor; }
        }

        private double bias = 0.0;
        public override double Bias
        {
            get { return bias; }
        }

        private double[,] filterMatrix =
            new double[,] { {  0, -2,  0, }, 
                            { -2, 11, -2, }, 
                            {  0, -2,  0, }, };

        public override double[,] FilterMatrix
        {
            get { return filterMatrix; }
        }
        #endregion
    }

    public class Sharpen5x5Filter : ConvolutionFilterBase
    {
        #region Sharpen5x5Filter : ConvolutionFilterBase

        public override string FilterName
        {
            get { return "Sharpen5x5Filter"; }
        }

        private double factor = 1.0 / 8.0;
        public override double Factor
        {
            get { return factor; }
        }

        private double bias = 0.0;
        public override double Bias
        {
            get { return bias; }
        }

        private double[,] filterMatrix =
            new double[,] { { -1, -1, -1, -1, -1, }, 
                            { -1,  2,  2,  2, -1, }, 
                            { -1,  2,  8,  2,  1, },
                            { -1,  2,  2,  2, -1, },
                            { -1, -1, -1, -1, -1, }, };

        public override double[,] FilterMatrix
        {
            get { return filterMatrix; }
        }
        #endregion
    }

    public class IntenseSharpenFilter : ConvolutionFilterBase
    {
        #region IntenseSharpenFilter : ConvolutionFilterBase

        public override string FilterName
        {
            get { return "IntenseSharpenFilter"; }
        }

        private double factor = 1.0;
        public override double Factor
        {
            get { return factor; }
        }

        private double bias = 0.0;
        public override double Bias
        {
            get { return bias; }
        }

        private double[,] filterMatrix =
            new double[,] { { 1,  1,  1, }, 
                            { 1, -7,  1, }, 
                            { 1,  1,  1, }, };

        public override double[,] FilterMatrix
        {
            get { return filterMatrix; }
        }
        #endregion
    }

    public class Usredniajacy : ConvolutionFilterBase      // FiltryWygladzajace
    {
        #region Usredniajacy : ConvolutionFilterBase

        public override string FilterName
        {
            get { return "Usredniajacy"; }      // "SoftenFilter"
        }

        private double factor = 1.0 / 9.0;
        public override double Factor
        {
            get { return factor; }
        }

        private double bias = 0.0;
        public override double Bias
        {
            get { return bias; }
        }

        private double[,] filterMatrix =
            new double[,] { { 1, 1, 1, }, 
                            { 1, 1, 1, }, 
                            { 1, 1, 1, }, };

        public override double[,] FilterMatrix
        {
            get { return filterMatrix; }
        }
        #endregion
    }

    public class Kwadratowy : ConvolutionFilterBase      // FiltryWygladzajace
    {
        #region Kwadratowy : ConvolutionFilterBase

        public override string FilterName
        {
            get { return "Kwadratowy"; }      // 
        }

        private double factor = 1.0 / 25.0;   // 24.0
        public override double Factor
        {
            get { return factor; }
        }

        private double bias = 0.0;
        public override double Bias
        {
            get { return bias; }
        }

        private double[,] filterMatrix =
            new double[,] { { 1, 1, 1, 1, 1 }, 
                            { 1, 1, 1, 1, 1 }, 
                            { 1, 1, 1, 1, 1 }, 
                            { 1, 1, 1, 1, 1 }, 
                            { 1, 1, 1, 1, 1 }, };

        public override double[,] FilterMatrix
        {
            get { return filterMatrix; }
        }
        #endregion
    }

    public class Kolowy : ConvolutionFilterBase        // FiltryWygladzajace
    {
        #region Kolowy : ConvolutionFilterBase

        public override string FilterName
        {
            get { return "Kolowy"; }       
        }

        private double factor = 1.0 / 21.0;   
        public override double Factor
        {
            get { return factor; }
        }

        private double bias = 0.0;
        public override double Bias
        {
            get { return bias; }
        }

        private double[,] filterMatrix =
            new double[,] { { 0, 1, 1, 1, 0 }, 
                            { 1, 1, 1, 1, 1 }, 
                            { 1, 1, 1, 1, 1 }, 
                            { 1, 1, 1, 1, 1 }, 
                            { 0, 1, 1, 1, 0 }, };

        public override double[,] FilterMatrix
        {
            get { return filterMatrix; }
        }
        #endregion
    }

    public class Piramidalny : ConvolutionFilterBase      // FiltryWygladzajace
    {
        #region Piramidalny : ConvolutionFilterBase

        public override string FilterName
        {
            get { return "Piramidalny"; }      // 
        }

        private double factor = 1.0 / 81.0;   // 24.0
        public override double Factor
        {
            get { return factor; }
        }

        private double bias = 0.0;
        public override double Bias
        {
            get { return bias; }
        }

        private double[,] filterMatrix =
            new double[,] { { 1, 2, 3, 2, 1 }, 
                            { 2, 4, 6, 4, 2 }, 
                            { 3, 6, 9, 6, 3 }, 
                            { 2, 4, 6, 4, 2 }, 
                            { 1, 2, 3, 2, 1 }, };

        public override double[,] FilterMatrix
        {
            get { return filterMatrix; }
        }
        #endregion
    }

    public class Stozkowy : ConvolutionFilterBase      // FiltryWygladzajace
    {
        #region Stozkowy : ConvolutionFilterBase

        public override string FilterName
        {
            get { return "Stozkowy"; }      // 
        }

        private double factor = 1.0 / 80.0;   // 24.0
        public override double Factor
        {
            get { return factor; }
        }

        private double bias = 0.0;
        public override double Bias
        {
            get { return bias; }
        }

        private double[,] filterMatrix =
            new double[,] { { 0, 0, 1, 0, 0 }, 
                            { 0, 2, 2, 2, 0 }, 
                            { 1, 2, 5, 2, 1 }, 
                            { 0, 2, 2, 2, 0 }, 
                            { 0, 0, 1, 0, 0 }, };

        public override double[,] FilterMatrix
        {
            get { return filterMatrix; }
        }
        #endregion
    }

    public class Gaussa3x3 : ConvolutionFilterBase      // FiltryWygladzajace
    {
        #region Gaussa3x3 : ConvolutionFilterBase

        public override string FilterName
        {
            get { return "Gaussa 3x3"; }
        }

        private double factor = 1.0 / 16.0;   // 8.0
        public override double Factor
        {
            get { return factor; }
        }

        private double bias = 0.0;
        public override double Bias
        {
            get { return bias; }
        }

        private double[,] filterMatrix =
            new double[,] { { 1, 2, 1 }, 
                            { 2, 4, 2 }, 
                            { 1, 2, 1 }, };

        public override double[,] FilterMatrix
        {
            get { return filterMatrix; }
        }
        #endregion
    }

    public class Gaussa5x5_1 : ConvolutionFilterBase      // FiltryWygladzajace
    {
        #region Gaussa5x5_1 : ConvolutionFilterBase

        public override string FilterName
        {
            get { return "Gaussa 5x5 1"; }
        }

        private double factor = 1.0 / 52.0;   // 8.0
        public override double Factor
        {
            get { return factor; }
        }

        private double bias = 0.0;
        public override double Bias
        {
            get { return bias; }
        }

        private double[,] filterMatrix =
            new double[,] { { 1, 1, 2, 1, 1 }, 
                            { 1, 2, 4, 2, 1 }, 
                            { 2, 4, 8, 4, 2 }, 
                            { 1, 2, 4, 2, 1 }, 
                            { 1, 1, 2, 1, 1 }, };

        public override double[,] FilterMatrix
        {
            get { return filterMatrix; }
        }
        #endregion
    }

    public class Gaussa5x5_2 : ConvolutionFilterBase      // FiltryWygladzajace
    {
        #region  Gaussa5x5_2 : ConvolutionFilterBase

        public override string FilterName
        {
            get { return "Gaussa 5x5 2"; }
        }

        private double factor = 1.0 / 25.0;   // 8.0
        public override double Factor
        {
            get { return factor; }
        }

        private double bias = 0.0;
        public override double Bias
        {
            get { return bias; }
        }

        private double[,] filterMatrix =
            new double[,] { { 0, 1, 2, 1, 0 }, 
                            { 1, 4, 8, 4, 1 }, 
                            { 2, 8, 16, 8, 2 }, 
                            { 1, 4, 8, 4, 1 }, 
                            { 0, 1, 2, 1, 0 }, };

        public override double[,] FilterMatrix
        {
            get { return filterMatrix; }
        }
        #endregion
    }

    public class Gaussa5x5_3 : ConvolutionFilterBase      // FiltryWygladzajace
    {
        #region Gaussa5x5_3 : ConvolutionFilterBase

        public override string FilterName
        {
            get { return "Gaussa 5x5 3"; }
        }

        private double factor = 1.0 / 273.0;   // 8.0
        public override double Factor
        {
            get { return factor; }
        }

        private double bias = 0.0;
        public override double Bias
        {
            get { return bias; }
        }

        private double[,] filterMatrix =
            new double[,] { { 1, 4, 7, 4, 1 }, 
                            { 4, 16, 26, 16, 4 }, 
                            { 7, 26, 41, 26, 7 }, 
                            { 4, 16, 26, 16, 4 }, 
                            { 1, 4, 7, 4, 1 }, };

        public override double[,] FilterMatrix
        {
            get { return filterMatrix; }
        }
        #endregion
    }

    public class Gaussa5x5_4 : ConvolutionFilterBase      // FiltryWygladzajace
    {
        #region Gaussa5x5_4 : ConvolutionFilterBase

        public override string FilterName
        {
            get { return "Gaussa 5x5 4"; }
        }

        private double factor = 1.0 / 159.0;   // 8.0
        public override double Factor
        {
            get { return factor; }
        }

        private double bias = 0.0;
        public override double Bias
        {
            get { return bias; }
        }

        public double[,] filterMatrix =
            new double[,] {{ 2,  4,  5,  4, 2  }, 
                           { 4,  9, 12,  9, 4  }, 
                           { 5, 12, 15, 12, 5  },
                           { 4,  9, 12,  9, 4  },
                           { 2,  4,  5,  4, 2  },};

        public override double[,] FilterMatrix
        {
            get { return filterMatrix; }
        }
        #endregion
    }

    public class Gaussa5x5_5 : ConvolutionFilterBase      // FiltryWygladzajace
    {
        #region Gaussa5x5_5 : ConvolutionFilterBase

        public override string FilterName
        {
            get { return "Gaussa 5x5 5"; }
        }

        private double factor = 1.0 / 256.0;   // 8.0
        public override double Factor
        {
            get { return factor; }
        }

        private double bias = 0.0;
        public override double Bias
        {
            get { return bias; }
        }

        private double[,] filterMatrix =
            new double[,] { {  1,   4,  6,  4,  1  }, 
                              {  4,  16, 24, 16,  4  }, 
                              {  6,  24, 36, 24,  6  },
                              {  4,  16, 24, 16,  4  },
                              {  1,   4,  6,  4,  1  }, };

        public override double[,] FilterMatrix
        {
            get { return filterMatrix; }
        }
        #endregion
    }

    public class Gaussa7x7 : ConvolutionFilterBase      // FiltryWygladzajace
    {
        #region  Gaussa7x7 : ConvolutionFilterBase

        public override string FilterName
        {
            get { return "Gaussa 7x7"; }
        }

        private double factor = 1.0 / 49.0;   // 8.0
        public override double Factor
        {
            get { return factor; }
        }

        private double bias = 0.0;
        public override double Bias
        {
            get { return bias; }
        }

        private double[,] filterMatrix =
            new double[,] { { 1, 1, 2, 2, 2, 1, 1 }, 
                            { 1, 2, 2, 4, 2, 2, 1 }, 
                            { 2, 2, 4, 8, 4, 2, 2 }, 
                            { 2, 4, 8, 16, 8, 4, 2 }, 
                            { 2, 2, 4, 8, 4, 2, 2 }, 
                            { 1, 2, 2, 4, 2, 2, 1 }, 
                            { 1, 1, 2, 2, 2, 1, 1 }, };

        public override double[,] FilterMatrix
        {
            get { return filterMatrix; }
        }
        #endregion
    }

    public class LP_1 : ConvolutionFilterBase      // FiltryWygladzajace
    {
        #region LP_1 : ConvolutionFilterBase

        public override string FilterName
        {
            get { return "LP 1"; }
        }

        private double factor = 1.0 / 10.0;   // 8.0
        public override double Factor
        {
            get { return factor; }
        }

        private double bias = 0.0;
        public override double Bias
        {
            get { return bias; }
        }

        private double[,] filterMatrix =
            new double[,] { { 1, 1, 1 }, 
                            { 1, 2, 1 }, 
                            { 1, 1, 1 } };

        public override double[,] FilterMatrix
        {
            get { return filterMatrix; }
        }
        #endregion
    }

    public class LP_2 : ConvolutionFilterBase      // FiltryWygladzajace
    {
        #region LP_2 : ConvolutionFilterBase

        public override string FilterName
        {
            get { return "LP 2"; }
        }

        private double factor = 1.0 / 12.0;   // 8.0
        public override double Factor
        {
            get { return factor; }
        }

        private double bias = 0.0;
        public override double Bias
        {
            get { return bias; }
        }

        private double[,] filterMatrix =
            new double[,] { { 1, 1, 1 }, 
                            { 1, 4, 1 }, 
                            { 1, 1, 1 } };

        public override double[,] FilterMatrix
        {
            get { return filterMatrix; }
        }
        #endregion 
    }

    public class LP_3 : ConvolutionFilterBase      // FiltryWygladzajace
    {
        #region LP_3 : ConvolutionFilterBase

        public override string FilterName
        {
            get { return "LP 3"; }
        }

        private double factor = 1.0 / 21.0;   // 8.0
        public override double Factor
        {
            get { return factor; }
        }

        private double bias = 0.0;
        public override double Bias
        {
            get { return bias; }
        }

        private double[,] filterMatrix =
            new double[,] { { 1, 1 , 1 }, 
                            { 1, 12, 1 }, 
                            { 1, 1 , 1 } };

        public override double[,] FilterMatrix
        {
            get { return filterMatrix; }
        }
        #endregion
    }

    public class Kwadratowy2 : ConvolutionFilterBase      // FiltryWygladzajace
    {
        #region Kwadratowy2 : ConvolutionFilterBase

        public override string FilterName
        {
            get { return "Kwadratowy2"; }      // 
        }

        private double factor = 1.0 / 25.0;   // 24.0
        public override double Factor
        {
            get { return factor; }
        }

        private double bias = 5.0;
        public override double Bias
        {
            get { return bias; }
        }

        private double[,] filterMatrix =
            new double[,] { { 1, 1, 1, 1, 1 }, 
                            { 1, 1, 1, 1, 1 }, 
                            { 1, 1, 1, 1, 1 }, 
                            { 1, 1, 1, 1, 1 }, 
                            { 1, 1, 1, 1, 1 }, };

        public override double[,] FilterMatrix
        {
            get { return filterMatrix; }
        }
        #endregion
    }

    public class EdgeDetectionFilter : ConvolutionFilterBase
    {
        #region EdgeDetectionFilter : ConvolutionFilterBase

        public override string FilterName
        {
            get { return "EdgeDetectionFilter"; }
        }

        private double factor = 1.0;
        public override double Factor
        {
            get { return factor; }
        }

        private double bias = 0.0;
        public override double Bias
        {
            get { return bias; }
        }

        private double[,] filterMatrix =
            new double[,] { { -1, -1, -1, }, 
                            { -1,  8, -1, }, 
                            { -1, -1, -1, }, };

        public override double[,] FilterMatrix
        {
            get { return filterMatrix; }
        }
        #endregion 
    }

    public class EdgeDetection45DegreeFilter : ConvolutionFilterBase
    {
        #region  EdgeDetection45DegreeFilter : ConvolutionFilterBase

        public override string FilterName
        {
            get { return "EdgeDetection45DegreeFilter"; }
        }

        private double factor = 1.0;
        public override double Factor
        {
            get { return factor; }
        }

        private double bias = 0.0;
        public override double Bias
        {
            get { return bias; }
        }

        private double[,] filterMatrix =
            new double[,] { { -1,  0,  0,  0,  0, }, 
                            {  0, -2,  0,  0,  0, }, 
                            {  0,  0,  6,  0,  0, },
                            {  0,  0,  0, -2,  0, },
                            {  0,  0,  0,  0, -1, }, };

        public override double[,] FilterMatrix
        {
            get { return filterMatrix; }
        }
        #endregion
    }

    public class HorizontalEdgeDetectionFilter : ConvolutionFilterBase
    {
        #region HorizontalEdgeDetectionFilter : ConvolutionFilterBase

        public override string FilterName
        {
            get { return "HorizontalEdgeDetectionFilter"; }
        }

        private double factor = 1.0;
        public override double Factor
        {
            get { return factor; }
        }

        private double bias = 0.0;
        public override double Bias
        {
            get { return bias; }
        }

        private double[,] filterMatrix =
            new double[,] { {  0,  0,  0,  0,  0, }, 
                            {  0,  0,  0,  0,  0, }, 
                            { -1, -1,  2,  0,  0, },
                            {  0,  0,  0,  0,  0, },
                            {  0,  0,  0,  0,  0, }, };

        public override double[,] FilterMatrix
        {
            get { return filterMatrix; }
        }
        #endregion
    }

    public class VerticalEdgeDetectionFilter : ConvolutionFilterBase
    {
        #region VerticalEdgeDetectionFilter : ConvolutionFilterBase

        public override string FilterName
        {
            get { return "VerticalEdgeDetectionFilter"; }
        }

        private double factor = 1.0;
        public override double Factor
        {
            get { return factor; }

        }

        private double bias = 0.0;
        public override double Bias
        {
            get { return bias; }
        }

        private double[,] filterMatrix =
            new double[,] { {  0,  0, -1,  0,  0, }, 
                            {  0,  0, -1,  0,  0, }, 
                            {  0,  0,  4,  0,  0, },
                            {  0,  0, -1,  0,  0, },
                            {  0,  0, -1,  0,  0, }, };

        public override double[,] FilterMatrix
        {
            get { return filterMatrix; }
        }
        #endregion
    }

    public class EdgeDetectionTopLeftBottomRightFilter : ConvolutionFilterBase
    {
        #region EdgeDetectionTopLeftBottomRightFilter : ConvolutionFilterBase

        public override string FilterName
        {
            get { return "EdgeDetectionTopLeftBottomRightFilter"; }
        }

        private double factor = 1.0;
        public override double Factor
        {
            get { return factor; }
        }

        private double bias = 0.0;
        public override double Bias
        {
            get { return bias; }
        }

        private double[,] filterMatrix =
            new double[,] { { -5,  0,  0, }, 
                            {  0,  0,  0, }, 
                            {  0,  0,  5, }, };

        public override double[,] FilterMatrix
        {
            get { return filterMatrix; }
        }
        #endregion
    }

}
