﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.IO;
using System.Diagnostics;

namespace APO_BB_DK_v012013
{
    // klasa obiektów przechowujących bitmapę
    public class MyBitmap : IDisposable
    {
        private Bitmap bitmap;
        private BitmapData bitmapData;
        private int stride;
        private int width;
        private int height;
     //   private MyBitmap myBitmap;


           // zwraca lub zapisuje bitmapę w obiekcie
        public Bitmap Bitmap
        {
            get 
            { 
                return bitmap; 
            }
            set 
            { 
                bitmap = value; 
            }
        }

        // zwraca szerokość bitmapy (ilość pikseli)
        public int Width
        {
            get
            {
                return width;
            }
        }

        // zwraca wysokość bitmapy (ilość pikseli)
        public int Height
        {
            get
            {
                return height;
            }
        }

        // zwraca rozmiar bitmapy (obiekt typu Size)
        public Size Size
        {
            get
            {
                return new Size(Width, Height);
            }
        }

        // zwraca ilość bajtów na piksel
        public static int BytesPerPixel
        {
            get { return 4; }
        }


        


        // zwraca lub ustawia kolor piksela o zadanym położeniu
        public Color this[int x, int y]
        {
            get
            {
                while (x < 0)
                    x += Width;
                while (x >= Width)
                    x -= Width;
                while (y < 0)
                    y += Height;
                while (y >= Height)
                    y -= Height;

                unsafe
                {
                    int* ptr = (int*)(((int*)bitmapData.Scan0) + (y * stride) + x);
                    return Color.FromArgb(*ptr);
                }
            }
            set
            {
                while (x < 0)
                    x += Width;
                while (x >= Width)
                    x -= Width;
                while (y < 0)
                    y += Height;
                while (y >= Height)
                    y -= Height;

                unsafe
                {
                    int* ptr = (int*)(((int*)bitmapData.Scan0) + (y * stride) + x);
                    *ptr = value.ToArgb();
                }
            }
        }

        // konstruktor
        // tworzy nową, pustą bitmapę o zadanych wymiarach
        public MyBitmap(int width, int height)
        {
            this.width = width;
            this.height = height;

            bitmap = new Bitmap(width, height, PixelFormat.Format32bppArgb);

            Lock();

            stride = bitmapData.Stride / BytesPerPixel;
        }

        // konstruktor
        // tworzy nowy obietk z wykorzystaniem podanej bitmapy
        public MyBitmap(Bitmap bitmap)
        {
            if (bitmap == null)
                throw new ArgumentNullException("bitmap");

            width = bitmap.Width;
            height = bitmap.Height;

            if (bitmap.PixelFormat == PixelFormat.Format32bppArgb)
                this.bitmap = bitmap;
            else
                this.bitmap = bitmap.Clone(new Rectangle(0, 0, width, height), PixelFormat.Format32bppArgb);

            Lock();

            stride = bitmapData.Stride / BytesPerPixel;
        }

  

        // rysuje bitmapę na podanej grafice
        public void Draw(Graphics graphics, int x, int y)
        {
            Unlock();
            graphics.DrawImage(bitmap, x, y);
            Lock();
        }

        // zapisuje bitmapę do podanego strumienia
        public void Save(Stream stream, ImageFormat format)
        {
            bitmap.Save(stream, format);
        }

        // zapisuje bitmapę do podanego pliku
        public void Save(string filename, ImageFormat format)
        {
            bitmap.Save(filename, format);
        }

        // zakłda blokadę na operacje
        public void Lock()
        {
            if (bitmapData != null)
                return;

            bitmapData = bitmap.LockBits(new Rectangle(0, 0, width, height), ImageLockMode.ReadWrite, PixelFormat.Format32bppArgb);
        }

        // zwalnia blokadę
        public Bitmap Unlock()
        {
            if (bitmapData == null)
                return bitmap;

            bitmap.UnlockBits(bitmapData);
            bitmapData = null;

            return bitmap;
        }

        // zarządzanie pamięcią
        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        // zarządzanie pamięcią
        protected virtual void Dispose(bool disposing)
        {
            if (disposing)
            {
                Unlock();

                if (bitmap != null)
                    bitmap.Dispose();
            }

            bitmapData = null;
            bitmap = null;
        }

        // zwraca obiekt typu Graphics z bitmapy
      //  public Graphics CreateGraphics()
      //  {
          //  return Graphics.ObrazMax(bitmap);
      //  }
    }
}
