using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

namespace APO.forms
{
    public partial class regulacja : UserControl
    {
        private Bitmap bitmapa;
        private BitmapProcessing.FastBitmap fbitmapa;
        private int [] values=null;
        private int maximum = 0;
        private bool initialized = false;
        private bool mouse_draw=false;

        private int draw_start = 0;
        private int draw_stop = 0;
        
        private double skala_wysokosci = 1.0f;
        private double skala_szerokosci = 1.0f;

        
        public delegate void UpdateHandler(object sender);
        public event UpdateHandler Zmiana;
        public int[] vals
        {
            get { return values; }
            set { }
        }
        public void init(int[] vals, int max)
        {
            values = vals;
            maximum = max;
            skala_wysokosci = (double)this.Height / (double)max;
            skala_szerokosci = 256.0f / (double)this.Width;
            initialized = true;
            
        }
        
        public regulacja()
        {
            InitializeComponent();
            
        }
        private void regulacja_Load(object sender, EventArgs e)
        {
            if (this.Width < 256)
                this.Width = 256;
            if (this.Height < 256)
                this.Height = 256;
            bitmapa = new Bitmap(this.Width, this.Height);
            fbitmapa = new BitmapProcessing.FastBitmap(bitmapa);
            
            update_curve();
        }
        
        private int height_safety_check(int val)
        {
            if (val >= bitmapa.Height)
                return bitmapa.Height-1;
            else if (val < 0)
                return 0;
            return val;
        }
        private int width_safety_check(int val)
        {
            if (val >= bitmapa.Width)
                return bitmapa.Width - 1;
            else if (val < 1)
                return 1;
            return val;
        }
        private int vals_safety_check(int val)
        {
            if (val > 255)
                return 255;
            else if (val < 1)
                return 1;
            return val;
        }
        private void update_curve()
        {
            if (fbitmapa != null && initialized)
            {
                update_curve_logic();
                Zmiana(this);
            }
        }
        public void update_curve_no_event()
        {
            if (fbitmapa != null && initialized)
            {
                update_curve_logic();
                
            }
        }
        private void update_curve_logic()
        {
            
                fbitmapa.LockImage();
                for (int x = 0; x < this.Width; x++)
                {
                    for (int y = 0; y < this.Height; y++)
                    {
                        fbitmapa.SetPixel(x, y, Color.White);
                    }
                }
                fbitmapa.SetPixel(0, height_safety_check(this.Height - 1 - (int)(values[0] * skala_wysokosci)), Color.Brown);
                for (int x = 1; x < this.Width; x++)
                {
                    //special variable for referencing variables array (scaling)
                    int vals_x = vals_safety_check((int)((double)x * skala_szerokosci));


                    if (values[vals_x - 1] < values[vals_x])
                    {
                        for (int y = height_safety_check(this.Height - 1 - (int)(values[vals_x - 1] * skala_wysokosci)); y > height_safety_check(this.Height - 1 - (int)(values[vals_x] * skala_wysokosci)); y--)
                        {
                            fbitmapa.SetPixel(x, y, Color.Brown);
                        }
                    }
                    else
                    {
                        for (int y = height_safety_check(this.Height - 1 - (int)(values[vals_x - 1] * skala_wysokosci)); y <= height_safety_check(this.Height - 1 - (int)(values[vals_x] * skala_wysokosci)); y++)
                        {
                            fbitmapa.SetPixel(x, y, Color.Brown);
                        }
                    }

                }
                fbitmapa.UnlockImage();
                krzywa.Image = bitmapa;
                
            
        }
        private void update_curve_no_clear()
        {
            if (fbitmapa != null && initialized)
            {
                fbitmapa.LockImage();
                
                int tmp_draw_start=width_safety_check(Math.Min(draw_start,draw_stop));
                int tmp_draw_stop=width_safety_check(Math.Max(draw_start,draw_stop));
                //fbitmapa.SetPixel(0, this.Height - 1 - (int)(values[0] * skala_wysokosci), Color.Brown);
                for (int x = tmp_draw_start; x < tmp_draw_stop; x++)
                {
                    //special variable for referencing variables array (scaling)
                    int vals_x = vals_safety_check((int)((double)x * skala_szerokosci));
                    int st = height_safety_check(this.Height - 1 - (int)(values[vals_x - 1] * skala_wysokosci));
                    int ko = height_safety_check(this.Height - 1 - (int)(values[vals_x] * skala_wysokosci));
                    if (Math.Abs(ko - st) > 20)
                    {
                    }
                    else
                    {
                        if (values[vals_x - 1] < values[vals_x])
                        {
                            for (int y = st; y > ko; y--)
                            {
                                fbitmapa.SetPixel(x, y, Color.Green);
                            }
                        }
                        else
                        {
                            for (int y = st; y <= ko; y++)
                            {
                                fbitmapa.SetPixel(x, y, Color.Green);
                            }
                        }
                    }

                }
                fbitmapa.UnlockImage();
                krzywa.Image = bitmapa;
            }
        }
        private void krzywa_MouseClick(object sender, MouseEventArgs e)
        {
            
            //MessageBox.Show(e.X+"x"+e.Y);
            if (initialized)
            {
                int vals_x=vals_safety_check((int)((double)e.X*skala_szerokosci));
                values[vals_x] = (int)((double)(this.Height - e.Y) / skala_wysokosci);
               
                values[vals_x - 1] = values[vals_x] - ((values[vals_x] - values[vals_x - 1]) / 2);
                if (vals_x < 255)
                    values[vals_x + 1] = values[vals_x] - ((values[vals_x] - values[vals_x + 1]) / 2);
                
                update_curve();
            }
        }

        private void krzywa_MouseDown(object sender, MouseEventArgs e)
        {
            if (initialized)
            {
                mouse_draw = true;
                draw_start = e.X;
            }
        }

        private void krzywa_MouseMove(object sender, MouseEventArgs e)
        {
            
            if (mouse_draw&& initialized)
            {
                //teoretycznie nonsensem jest sprawdza, czy obiekt moe zwrci
                //ujemne wartosci pooenia myszy w jego obrbie
                //w praktyce trzeba pamita, e przy wcinitym wczeniej przycisku
                //mousemove dziaa take poza obiektem, zwracajc ujemne wartoci..
                if (e.X >= 0 && e.X < this.Width)
                {
                    draw_stop = e.X;
                    int vals_x = vals_safety_check((int)((double)e.X * skala_szerokosci));
                    values[vals_x] = vals_safety_check((int)((double)(this.Height - e.Y) / skala_wysokosci));
                    if (vals_x > 0)
                        values[vals_x - 1] = vals_safety_check((int)((double)(this.Height - e.Y) / skala_wysokosci));
                    //if (vals_x > 1)
                     //   values[vals_x - 2] = (int)((double)(this.Height - e.Y) / skala_wysokosci);
                    if (vals_x < 255)
                        values[vals_x + 1] = vals_safety_check((int)((double)(this.Height - e.Y) / skala_wysokosci));
                    //if (vals_x < 254)
                        //values[vals_x + 2] = (int)((double)(this.Height - e.Y) / skala_wysokosci);

                    update_curve_no_clear();
                }
                else if(e.Y<0|| e.Y>=this.Height)
                {
                    update_curve();
                }
                else
                {
                    update_curve();
                }
                
            }
        }

        private void krzywa_MouseUp(object sender, MouseEventArgs e)
        {
            if (initialized)
            {
                mouse_draw = false;
                update_curve();
            }
            
        }
        

        

        

        
    }
}
